// ---------------------------------------------------------------------
/**
 Copyright (c) 2004-2006 Honda Research Institute Japan, Co., Ltd.

 \file ArrayInit.h
 \brief \~English Header file for initializing arrays (Microphone Array System)
 \brief \~Japanese z܂ލ\̂wb_t@C(Microphone Array System)
 \author Natsu Tanaka
 \date 20 Jul 2005
 \version $Id: ArrayInit.h 3538 2012-02-29 07:25:35Z tanaka $
*/

// ---------------------------------------------------------------------
#ifndef	ArrayInit_h_defined
#define	ArrayInit_h_defined
// ---------------------------------------------------------------------
#include	"MICARY_struct.h"
// ---------------------------------------------------------------------

// 2007/06/11 added by N.Tanaka C++CĂяo
#ifdef __cplusplus
extern "C" {
#endif

/**
 \brief \~English It allocates float array
 \brief \~Japanese HA_Floatz̗̈mۂ
 \param array \English It's floatArray
 \~Japanese HA_Floatz܂ލ\
 \param iArraySize \~English It's the size of array
 \~Japanese mۂvf
*/
void FloatArrayAlloc(MICARY_FloatArray * array, int iArraySize);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates float array
 \brief \~Japanese HA_Floatz̗̈mۂ
 \param array \English It's floatArray
 \~Japanese HA_Floatz܂ލ\
 \param iArraySize \~English It's the size of array in use
 \~Japanese gpvf
 \param iReservedSize \~English It's the size of array
 \~Japanese mۂvf
*/
void FloatArrayAlloc_r(MICARY_FloatArray * array, int iArraySize, int iReservedSize);
#endif

/**
 \brief \~English It frees float array
 \brief \~Japanese HA_Floatz 
 \param array \~English FloatArray
 \~Japanese HA_Floatz܂ލ\
*/
void FloatArrayFree(MICARY_FloatArray * array);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees float array
 \brief \~Japanese HA_Floatz̎gpTCYNA 
 \param array \~English FloatArray
 \~Japanese HA_Floatz܂ލ\
*/
void FloatArrayClear(MICARY_FloatArray * array);
#endif

/**
 \brief \~English It allocates int array
 \brief \~Japanese HA_Integerz̗̈mۂ
 \param array \~English It's intArray
 \~Japanese HA_Integerz܂ލ\
 \param iArraySize \~English It's the size of array
 \~Japanese mۂvf
*/
void IntArrayAlloc(MICARY_IntArray * array, int iArraySize);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates int array
 \brief \~Japanese HA_Intz̗̈mۂ
 \param array \English It's intArray
 \~Japanese HA_Floatz܂ލ\
 \param iArraySize \~English It's the size of array in use
 \~Japanese gpvf
 \param iReservedSize \~English It's the size of array
 \~Japanese mۂvf
*/
void IntArrayAlloc_r(MICARY_IntArray * array, int iArraySize, int iReservedSize);
#endif

/**
 \brief It frees int array
 \brief \~Japanese HA_Integerz 
 \param array \~English IntArray
 \~Japanese HA_Integerz܂ލ\
*/
void IntArrayFree(MICARY_IntArray * array);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees int array
 \brief \~Japanese HA_Intz̎gpTCYNA 
 \param array \~English IntArray
 \~Japanese HA_Intz܂ލ\
*/
void IntArrayClear(MICARY_IntArray * array);
#endif

/**
 \brief \~English It allocates complex array
 \brief \~Japanese HA_Complexz̗̈mۂ
 \param array \~English It's ComplexArray
 \~Japanese HA_Complexz܂ލ\
 \param iArraySize \~English It's the size of array
 \~Japanese mۂvf
*/
void ComplexArrayAlloc(MICARY_ComplexArray * array, int iArraySize);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates complex array
 \brief \~Japanese HA_Complexz̗̈mۂ
 \param array \English It's complexArray
 \~Japanese HA_Complexz܂ލ\
 \param iArraySize \~English It's the size of array in use
 \~Japanese gpvf
 \param iReservedSize \~English It's the size of array
 \~Japanese mۂvf
*/
void ComplexArrayAlloc_r(MICARY_ComplexArray * array, int iArraySize, int iReservedSize);
#endif

/**
 \brief \~English It frees complex array
 \brief \~Japanese HA_Complexz 
 \param array \~English It's ComplexArray
 \~Japanese HA_Complexz܂ލ\
*/
void ComplexArrayFree(MICARY_ComplexArray * array);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees complex array
 \brief \~Japanese HA_Complexz̎gpTCYNA 
 \param array \~English ComplexArray
 \~Japanese HA_Complexz܂ލ\
*/
void ComplexArrayClear(MICARY_ComplexArray * array);
#endif

/**
 \brief \~English It allocates location array
 \brief \~Japanese MICARY_Locationzmۂ
 \param array \~English It's LocationArray
 \~Japanese MICARY_Locationz܂ލ\
 \param iArraySize It's the size of array
 \~Japanese mۂvf
*/
void LocationArrayAlloc(MICARY_LocationArray * array, int iArraySize);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates location array
 \brief \~Japanese MICARY_Locationzmۂ
 \param array \~English It's LocationArray
 \~Japanese MICARY_Locationz܂ލ\
 \param iArraySize \~English It's the size of array in use
 \~Japanese gpvf
 \param iReservedSize \~English It's the size of array
 \~Japanese mۂvf
*/
void LocationArrayAlloc_r(MICARY_LocationArray * array, int iArraySize, int iReservedSize);
#endif

/**
 \brief It frees location array
 \brief \~Japanese MICARY_Locationz
 \param array It's LocationArray
 \~Japanese MICARY_Locationz܂ލ\
*/
void LocationArrayFree(MICARY_LocationArray * array);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees location array
 \brief \~Japanese MICARY_Locationz̎gpTCYNA 
 \param array \~English LocationArray
 \~Japanese MICARY_Locationz܂ލ\
*/
void LocationArrayClear(MICARY_LocationArray * array);
#endif

/**
 \brief \~English It allocates database of beamforming
 \brief \~Japanese BFWi[zmۂ
 \param BFD \~English It's Database of beamforming
 \~Japanese BFWi[z܂ލ\\n
 [ʒu[}CNzʒu[gC]]̏Ɋi[\n
 (ʒus, }CNʒum, gCf̏ꍇ):\n
 BF(S1M1F1), BF(S1M1F2), ..., BF(S1M1Ff), BF(S1M2F1), ..., BF(S1MmFf), BF(S2M1F1), ..., BF(SsMmFf)
 \param iFreqSize \~English It's the number of frequency Line
 \~Japanese BFW̎gC
 \param iLocSize \~English It's the number of location
 \~Japanese BFW̉ʒu
 \param iMicSize \~English It's the number of microphone
 \~Japanese BFW̃}CNzʒu
*/
void BFDatabaseAlloc(MICARY_BFDatabase * BFD, int iFreqSize, int iLocSize, int iMicSize);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates database of beamforming
 \brief \~Japanese BFWi[zmۂ
 \param BFD \~English It's Database of beamforming
 \~Japanese BFWi[z܂ލ\\n
 [ʒu[}CNzʒu[gC]]̏Ɋi[\n
 (ʒus, }CNʒum, gCf̏ꍇ):\n
 BF(S1M1F1), BF(S1M1F2), ..., BF(S1M1Ff), BF(S1M2F1), ..., BF(S1MmFf), BF(S2M1F1), ..., BF(SsMmFf)
 \param iFreqSize \~English It's the number of frequency Line
 \~Japanese BFW̎gC
 \param iLocSize \~English It's the number of location
 \~Japanese BFW̉ʒu
 \param iMicSize \~English It's the number of microphone
 \~Japanese BFW̃}CNzʒu
 \param iReservedFreqSize \~English It's the size of array
 \~Japanese mۂvf(Freq)
 \param iReservedLocSize \~English It's the size of array
 \~Japanese mۂvf(Location)
 \param iReservedMicSize \~English It's the size of array
 \~Japanese mۂvf(Mic)
*/
void BFDatabaseAlloc_r(MICARY_BFDatabase * BFD, int iFreqSize, int iLocSize, int iMicSize, 
											  int iReservedFreqSize, int iReservedLocSize, int iReservedMicSize);
#endif

/**
 \brief \~English It frees database of beamforming
 \brief \~Japanese BFWi[zJ
 \param BFD \English It's Database of beamforming
 \~Japanese BFWi[z܂ލ\\n
*/
void BFDatabaseFree(MICARY_BFDatabase * BFD);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees database of beamforming
 \brief \~Japanese BFWi[z̎gpTCYNA 
 \param BFD \English It's Database of beamforming
 \~Japanese BFWi[z܂ލ\\n
*/
void BFDatabaseClear(MICARY_BFDatabase * BFD);
#endif

/**
 \brief \~English It allocates database of transfer function
 \brief \~Japanese `B֐i[zmۂ
 \param TFD \~English It's Database of transfer function
 \~Japanese `B֐i[z܂ލ\
 \param iFreqSize \~English It's the number of frequency Line
 \~Japanese `B֐̎gC
 \param iLocSize \~English It's the number of location
 \~Japanese `B֐̉ʒu
 \param iMicSize \~English It's the number of microphone
 \~Japanese `B֐̃}CNzʒu
*/
void TFDatabaseAlloc(MICARY_TFDatabase * TFD, int iFreqSize, int iLocSize, int iMicSize);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates database of transfer function
 \brief \~Japanese `B֐i[zmۂ
 \param TFD \~English It's Database of transfer function
 \~Japanese `B֐i[z܂ލ\
 \param iFreqSize \~English It's the number of frequency Line
 \~Japanese `B֐̎gC
 \param iLocSize \~English It's the number of location
 \~Japanese `B֐̉ʒu
 \param iMicSize \~English It's the number of microphone
 \~Japanese `B֐̃}CNzʒu
 \param iReservedFreqSize \~English It's the size of array
 \~Japanese mۂvf(Freq)
 \param iReservedLocSize \~English It's the size of array
 \~Japanese mۂvf(Location)
 \param iReservedMicSize \~English It's the size of array
 \~Japanese mۂvf(Mic)
*/
void TFDatabaseAlloc_r(MICARY_TFDatabase * TFD, int iFreqSize, int iLocSize, int iMicSize,
											  int iReservedFreqSize, int iReservedLocSize, int iReservedMicSize);
#endif

/**
 \brief \~English It frees database of transfer function
 \brief \~Japanese `B֐i[zJ
 \param TFD \~English It's Database of transfer function
 \~Japanese `B֐i[z܂ލ\
*/
void TFDatabaseFree(MICARY_TFDatabase * TFD);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees database of transfer function
 \brief \~Japanese `B֐i[z̎gpTCYNA
 \param TFD \~English It's Database of transfer function
 \~Japanese `B֐i[z܂ލ\
*/
void TFDatabaseClear(MICARY_TFDatabase * TFD);
#endif

/**
 \brief \~English It allocates int array(dim:2)
 \brief \~Japanese int^̓񎟌zmۂ
 \param array \~English It's the array of integer. (iArraySize1 * iArraySize2)
 \~Japanese int^̓񎟌z̃|C^
 \param iArraySize1 \~English It's the size of array (dim:1)
 \~Japanese 1̔ž
 \param iArraySize2 \~English It's the size of array (dim:2)
 \~Japanese 2̔ž
*/
void IntArray2Alloc(MICARY_IntArray2 * array, int iArraySize1, int iArraySize2);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates int array(dim:2)
 \brief \~Japanese int^̓񎟌zmۂ
 \param array \~English It's the array of integer. (iArraySize1 * iArraySize2)
 \~Japanese int^̓񎟌z̃|C^
 \param iArraySize1 \~English It's the size of array (dim:1)
 \~Japanese 1̔ž
 \param iArraySize2 \~English It's the size of array (dim:2)
 \~Japanese 2̔ž
 \param iReservedArraySize1 \~English It's the size of array (dim:1)
 \~Japanese 1̔z̊mی
 \param iReservedArraySize2 \~English It's the size of array (dim:2)
 \~Japanese 2̔z̊mی
*/
void IntArray2Alloc_r(MICARY_IntArray2 * array, int iArraySize1, int iArraySize2,
											  int iReservedSize1, int iReservedSize2);
#endif

/**
 \brief \~English It frees int array(dim:2)
 \brief \~Japanese int^̓񎟌zJ
 \param array \~English It's the array of integer. (iArraySize1 * iArraySize2)
 \~Japanese int^̓񎟌z̃|C^
*/
void IntArray2Free(MICARY_IntArray2 * array);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees int array(dim:2)
 \brief \~Japanese int^̓񎟌z̎gpTCYNA
 \param array \~English It's the array of integer. (iArraySize1 * iArraySize2)
 \~Japanese int^̓񎟌z̃|C^
*/
void IntArray2Clear(MICARY_IntArray2 * array);
#endif

// 2006/10/12 added by N.Tanaka
/**
 \brief \~English It allocates complex spectrum array
 \brief \~Japanese XyNgf[^pHA_Complexz̗̈mۂ
 \param pcsSpec \~English It's complex spectrum array
 \~Japanese HA_Complex̃XyNgp\
 \param numCh \~English It's the number of channel.
 \~Japanese `l
 \param iLowerIndex \~English It's the lower frequency index.
 \~Japanese gCfbNX
 \param iUpperIndex \~English It's the upper frequency index.
 \~Japanese gCfbNX
*/
void AllocComplexSpecArray(MICARY_ComplexSpecArray *pcsSpec,
						  HA_Num numCh,
						  HA_Integer iLowerIndex,
						  HA_Integer iUpperIndex);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates complex spectrum array
 \brief \~Japanese XyNgf[^pHA_Complexz̗̈mۂ
 \param pcsSpec \~English It's complex spectrum array
 \~Japanese HA_Complex̃XyNgp\
 \param numCh \~English It's the number of channel.
 \~Japanese `l
 \param iLowerIndex \~English It's the lower frequency index.
 \~Japanese gCfbNX
 \param iUpperIndex \~English It's the upper frequency index.
 \~Japanese gCfbNX
 \param iReservedChSize \~English It's the size of array
 \~Japanese z̊mی(ch)
 \param iReservedFreqSize \~English It's the size of array
 \~Japanese z̊mی(Freq)
*/
void AllocComplexSpecArray_r(MICARY_ComplexSpecArray *pcsSpec,
						  HA_Num numCh,
						  HA_Integer iLowerIndex,
						  HA_Integer iUpperIndex,
						  int iReservedChSize,
						  int iReservedFreqSize);
#endif

// 2006/10/12 added by N.Tanaka
/**
 \brief \~English It frees complex spectrum array
 \brief \~Japanese XyNgf[^pHA_Complexz̗̈J
 \param pcsSpec \~English It's complex spectrum array
 \~Japanese HA_Complex̃XyNgp\
*/
void FreeComplexSpecArray(MICARY_ComplexSpecArray *pcsSpec);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees complex spectrum array
 \brief \~Japanese XyNgf[^pHA_Complexz̎gpTCYNA
 \param pcsSpec \~English It's complex spectrum array
 \~Japanese HA_Complex̃XyNgp\
*/
void ClearComplexSpecArray(MICARY_ComplexSpecArray *pcsSpec);
#endif

// 2006/10/12 added by N.Tanaka
/**
 \brief \~English It allocates float spectrum array
 \brief \~Japanese XyNgf[^pHA_Floatz̗̈mۂ
 \param pfsSpec \~English It's float spectrum array
 \~Japanese HA_Float̃XyNgp\
 \param numCh \~English It's the number of channel.
 \~Japanese `l
 \param iLowerIndex \~English It's the lower frequency index.
 \~Japanese gCfbNX
 \param iUpperIndex \~English It's the upper frequency index.
 \~Japanese gCfbNX
*/
void AllocFloatSpecArray(MICARY_FloatSpecArray *pfsSpec,
						  HA_Num numCh,
						  HA_Integer iLowerIndex,
						  HA_Integer iUpperIndex);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It allocates freq spectrum array
 \brief \~Japanese XyNgf[^pHA_Floatz̗̈mۂ
 \param pfsSpec \~English It's float spectrum array
 \~Japanese HA_Float̃XyNgp\
 \param numCh \~English It's the number of channel.
 \~Japanese `l
 \param iLowerIndex \~English It's the lower frequency index.
 \~Japanese gCfbNX
 \param iUpperIndex \~English It's the upper frequency index.
 \~Japanese gCfbNX
 \param iReservedChSize \~English It's the size of array
 \~Japanese z̊mی(ch)
 \param iReservedFreqSize \~English It's the size of array
 \~Japanese z̊mی(Freq)
*/
void AllocFloatSpecArray_r(MICARY_FloatSpecArray *pfsSpec,
						  HA_Num numCh,
						  HA_Integer iLowerIndex,
						  HA_Integer iUpperIndex,
						  int iReservedChSize,
						  int iReservedFreqSize);
#endif


// 2006/10/12 added by N.Tanaka
/**
 \brief \~English It frees float spectrum array
 \brief \~Japanese XyNgf[^pHA_Floatz̗̈J
 \param pfsSpec \~English It's float spectrum array
 \~Japanese HA_Float̃XyNgp\
*/
void FreeFloatSpecArray(MICARY_FloatSpecArray *pfsSpec);
// 2011/11/16 added by N.Tanaka
#ifdef __MICARY_def_USE_RESERVED_ARRAY
/**
 \brief \~English It frees float spectrum array
 \brief \~Japanese XyNgf[^pHA_Floatz̎gpTCYNA
 \param pfsSpec \~English It's float spectrum array
 \~Japanese HA_Float̃XyNgp\
*/
void ClearFloatSpecArray(MICARY_FloatSpecArray *pfsSpec);
#endif

// 2007/08/22 added by N.Tanaka
/**
 \brief \~English It allocates char array(string)
 \brief \~Japanese charz(string)̗̈mۂ
 \param array \English It's CharArray(string)
 \~Japanese charz(string)܂ލ\
 \param iArraySize \~English It's the size of array
 \~Japanese mۂvf
*/
void StringAlloc(HA_String * array, int iArraySize);

// 2007/08/22 added by N.Tanaka
/**
 \brief \~English It frees char array(string)
 \brief \~Japanese charz(string) 
 \param array \~English CharArray(string)
 \~Japanese charz(string)܂ލ\
*/
void StringFree(HA_String * array);


#ifdef __cplusplus
}
#endif
// ---------------------------------------------------------------------
#endif	// ArrayInit_h_defined
