// ---------------------------------------------------------------------
/*
 * Copyright 2009 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */
// ---------------------------------------------------------------------
#include <stdlib.h>
#include <iostream>
#include <string.h>
#include <vector>
using namespace std;
// ---------------------------------------------------------------------
#include "Cexec.h"

// ---------------------------------------------------------------------
// RXgN^

Cexec::Cexec()
{
	m_psSepMat = (MICARY_SavedSeparation *)calloc(1,sizeof(MICARY_SavedSeparation)) ;	//sۑl
	SavedSeparationAlloc(m_psSepMat,0,0,0) ;
		

	LocationArrayAlloc(&locaSrcFixedNoise,0) ;
	LocationArrayAlloc(&locaSrcFixedNoise,1) ;
	m_psTFDB = NULL ;	
	m_psSMDB = NULL ;

	m_file = NULL ;
	m_file_FixedNoise = NULL ;
	m_file_sm = NULL ;
	m_file_tf = NULL ;
	
	LocationArrayAlloc(&locaMic,0) ;
	
	m_iFlgtf = 0 ;
	m_bFlgFixedNoise = false ;
	m_iFlgsm = 0 ;

	iMicPosShift = 1 ;	//default : FIX
	
	//m_updateW̊m
	m_updateW = (MICARY_UpdateSeparationMat *)calloc(1,sizeof(MICARY_UpdateSeparationMat)) ;
	UpdateSeparationMatAlloc(m_updateW,0,0,0) ;

	// m_laFrontSrcLoc̊m
	m_laFrontSrcLoc = (MICARY_LocationArray *)calloc(1,sizeof(MICARY_LocationArray)) ;
	LocationArrayAlloc(m_laFrontSrcLoc,0) ;

	// m_iUpdateMethodWex̊m
	m_iUpdateMethodWex = (MICARY_IntArray *)calloc(1,sizeof(MICARY_IntArray)) ;
	IntArrayAlloc(m_iUpdateMethodWex,0) ;

	// default value
	m_iFormat_TF_CONJ = __FILE_FORMAT_MICARY ;
	m_iFormat_InitW = __FILE_FORMAT_MICARY ;
	m_iFormat_ExportW = __FILE_FORMAT_MICARY ;
      m_iFormat_MicPos = __FILE_FORMAT_MICARY ;
	m_iFormat_FixedNoise = __FILE_FORMAT_MICARY ;

	// 2012/01/19 added
	LocationArrayAlloc(&m_plaSrcPosIni,0) ;
	ComplexArrayAlloc(&m_tmp_cmpaTFDB,0) ;

}

// ---------------------------------------------------------------------
// fXgN^
Cexec::~Cexec()
{
	if(m_psSepMat){
		SavedSeparationFree(m_psSepMat) ;
		free(m_psSepMat) ;
		m_psSepMat = NULL ;
	}
	if(&locaSrcFixedNoise){
       	LocationArrayFree(&locaSrcFixedNoise) ;
	}
	if(m_psTFDB){
		TFDatabaseFree(m_psTFDB) ;
		free(m_psTFDB) ;
		m_psTFDB = NULL ;
	}
	if(m_psSMDB){
		SMDatabaseFree(m_psSMDB) ;
		free(m_psSMDB) ;
		m_psSMDB = NULL ;
	}
	if(m_file){
		free(m_file) ;
		m_file = NULL ;
	}
	if(m_file_FixedNoise){
		free(m_file_FixedNoise) ;
		m_file_FixedNoise = NULL ;
	}
	if(m_file_sm){
		free(m_file_sm) ;
		m_file_sm = NULL ;
	}
	if(m_file_tf){
		free(m_file_tf) ;
		m_file_tf = NULL ;
	}
	if(&locaMic){
		LocationArrayFree(&locaMic) ;
	}
	m_iFlgtf = 0 ;
	m_bFlgFixedNoise = false ;
	m_iFlgsm = 0 ;

	if(m_updateW){
		UpdateSeparationMatFree(m_updateW) ;
		free(m_updateW) ;
		m_updateW = NULL ;
	}
	if(m_laFrontSrcLoc){
		LocationArrayFree(m_laFrontSrcLoc) ;
		free(m_laFrontSrcLoc) ;
		m_laFrontSrcLoc = NULL ;
	}

	if(m_iUpdateMethodWex){
		IntArrayFree(m_iUpdateMethodWex) ;
		free(m_iUpdateMethodWex) ;
		m_iUpdateMethodWex = NULL ;
	}

	// 2012/01/19 added
	LocationArrayFree(&m_plaSrcPosIni) ;
	ComplexArrayFree(&m_tmp_cmpaTFDB) ;
	
}

// ---------------------------------------------------------------------
// sۑl̃NA
int Cexec::Clear()
{
	SavedSeparationFree(m_psSepMat) ;
	return 1 ;
}

// ---------------------------------------------------------------------
//sۑlAgCA}CNAWz̊mۂƍ쐬
int Cexec::Initialize(const char *filename,float fspeed_sound,
							int iFlgTf,const char *filename_tf ,
							bool bFlgFixedNoise, const char *filename_FixedNoise,
							int iFlgSm,const char *filename_sm)
{
	this->Clear() ;
	//this->m_PrmStepSize.ha_fStepSizePrm=fStepPrm ;
 
	//}CNWt@C
	if(iFlgTf == 0){	//Calculation 
		this->m_file = (char *)malloc(sizeof(char)*strlen(filename)+1 ) ;
		strcpy(this->m_file,filename) ;
		if(Cexec ::ReadMicPos() == 0){
			std::cout << "Fail to open mic file." << std::endl ;
			m_iFlgtf = 0 ;
			m_bFlgFixedNoise = false ;
			m_iFlgsm = 0 ;
			
			return 0 ;
		}
		std::cout << "Read Mic position file." << std::endl ;
			
	}
	
	this->m_fspeed_sound = fspeed_sound ;
	//TF}gNXt@C
	if(iFlgTf == 1){	// Read data file
		this->m_file_tf = (char *)malloc(sizeof(char)*strlen(filename_tf)+1) ;
		strcpy(this->m_file_tf,filename_tf) ;
		this->m_iFlgtf = 1 ;
	}
	else{
		this->m_iFlgtf = 0 ;	//Calculation
	}
	this->m_bFlgFixedNoise = bFlgFixedNoise ;
	if(this->m_bFlgFixedNoise){
		if(!filename_FixedNoise){
			std::cout << "You must input FixedNoise position file." << std::endl ;
			m_iFlgtf = 0 ;
			m_bFlgFixedNoise = false ;
			m_iFlgsm = 0 ;
			return 0 ;
		}
		this->m_file_FixedNoise = (char *)malloc(sizeof(char)*strlen(filename_FixedNoise)+1) ;
		strcpy(this->m_file_FixedNoise,filename_FixedNoise) ;
		//ŏɓǂݍł
		int iMm,iMXm,iMYm,iMZm ;
	
		if(m_iFormat_FixedNoise == __FILE_FORMAT_HARK){
			if(!cHark.LoadPositionData(m_file_FixedNoise,&locaSrcFixedNoise,__FILE_TYPE_NOISE)){
		    		std ::cout << "Fail to Read FixedNoise position file." << std ::endl ;
				LocationArrayFree(&locaSrcFixedNoise) ;
				m_iFlgtf = 0 ;
				m_bFlgFixedNoise = false ;
				m_iFlgsm = 0 ;
				return 0 ;
			}
		}
			  
		std::cout << "Read FixedNoise position file." << std::endl ;
	}

	//s񏉊l(Calc or Read)
	if(iFlgSm == 1){
		this->m_file_sm = (char *)malloc(sizeof(char)*strlen(filename_sm)+1) ;
		strcpy(this->m_file_sm,filename_sm) ;
		this->m_iFlgsm = 1 ;
	}
	else{
		this->m_iFlgsm = 0 ;
	}
	return 1 ;
}
// ---------------------------------------------------------------------
// t[p[^̐ݒ
int Cexec ::SetPrmFrame(MICARY_PrmFrame *psPrmFrame ,int iFs ,
							int iLength,int iOverlap,int enumWin)
{
	psPrmFrame->ha_fFs = (float)iFs ;
	psPrmFrame->ha_iShift = (int)(iLength - iOverlap) ;
	psPrmFrame->ha_numNFFT = (HA_Num)iLength ;
	psPrmFrame->ha_numWindow = (HA_Num)iLength ;
	psPrmFrame->micary_nwnWindow = (MICARY_WindowEnum)enumWin ;
	return 1 ;
}
// ---------------------------------------------------------------------
//}CNWt@C̓ǂݍ݂Ɣz̍쐬
int Cexec ::ReadMicPos()
{
  	int iMm,iMXm,iMYm,iMZm ;
	if(m_iFormat_MicPos == __FILE_FORMAT_HARK){
		cout << "File open as HARK = " << m_file << endl ; 
	  	if(!cHark.LoadPositionData(m_file,&locaMic,__FILE_TYPE_MIC)){
	    		std ::cout << "Fail to Read mic position file." << std ::endl ;
			return 0 ;
	  	}
	}
	return 1 ;
}
// --------------------------------------------------------------------
// `B֐̍쐬(R)
int Cexec :: MakeTFMatrix(int iUpper_freq,MICARY_PrmFrame *psPrmFrame)
{
 
   if(CalcTFFreeH(&m_psSepMat->micary_tdTFDB,iUpper_freq,*psPrmFrame,m_psSepMat->micary_tdTFDB.micary_locaSource,
		m_psSepMat->micary_tdTFDB.micary_locaMic,(HA_Float)this->m_fspeed_sound) == 0)
	{
		std ::cout << "Fail to calculate TF-Matrix." << std::endl ;
		return 0 ;
	}

	return 1 ;
}

// --------------------------------------------------------------------
// `B֐̍쐬
int Cexec :: MakeTFMatrix2(MICARY_PrmFrame *psPrmFrame, int idsrc)
{
 
   if(CalcTFFreeH2(&m_psSepMat->micary_tdTFDB,*psPrmFrame,idsrc,(HA_Float)this->m_fspeed_sound) == 0)
	{
		std ::cout << "Fail to calculate TF-Matrix." << std::endl ;
		return 0 ;
	}

	return 1 ;
}

// --------------------------------------------------------------------
// `B֐t@C̓ǂݍ(Originalt@Cdl)
int Cexec :: ReadTFMatrix(MICARY_PrmFrame psPrmFrame)
{
	HA_String TFfile ;
	// Read
	TFfile.s = (HA_Char *)calloc(strlen(this->m_file_tf)+1,sizeof(char)) ;
	strcpy(TFfile.s,this->m_file_tf) ;
	TFfile.slen = strlen(this->m_file_tf) ;
	
	if(m_iFormat_TF_CONJ == __FILE_FORMAT_HARK){
		MICARY_TFDBFileHeader *psH ;
		psH = new MICARY_TFDBFileHeader() ;
		if(!cHark.ReadHeader(TFfile.s,psH)){
			std ::cout << "Fail to open TF data file." << std::endl ;
			if(TFfile.s){
				free(TFfile.s) ;
				TFfile.s = NULL ;
			}
			if(psH){
				delete psH ;
				psH = NULL ;
			}
			return 0 ;
		}
		if(m_psTFDB->micary_cmpaTF.ha_numValue != psH->lNSrc * psH->lNMic * psH->lNFreq){
		  TFDatabaseFree(m_psTFDB) ;
		  TFDatabaseAlloc(m_psTFDB,psH->lNFreq,psH->lNSrc,psH->lNMic) ;
		}

		if(!cHark.ReadInfo(TFfile.s,psH,
							&m_psTFDB->micary_locaSource ,
							&m_psTFDB->micary_locaMic,
							&m_psTFDB->micary_iaFreqLine)){
			std ::cout << "Fail to read TF data file." << std::endl ;
			if(TFfile.s){
				free(TFfile.s) ;
				TFfile.s = NULL ;
			}
			if(psH){
				delete psH ;
				psH = NULL ;
			}
			return 0 ;
		}
	

		if(!cHark.ReadData(TFfile.s,&m_psTFDB->micary_cmpaTF)){
			std ::cout << "Fail to read TF data file." << std::endl ;
			if(TFfile.s){
				free(TFfile.s) ;
				TFfile.s = NULL ;
			}
			if(psH){
				delete psH ;
				psH = NULL ;
			}
			return 0 ;
		}
		if(psH){
		  delete psH ;
		  psH = NULL ;
		}
	}
	if(TFfile.s){
		free(TFfile.s) ;
		TFfile.s = NULL ;
	}
	
	return 1 ;
}	
//-----------------------------------------------------------------------------
// TF Matrix̏
//TFf[^x[XAs񏉊lf[^x[X̃t@Cǂݍ
int Cexec :: InitTFMatrix(MICARY_PrmFrame *psPrmFrame)
{
	int iLower_f,iUpper_f,iNch,iNpos,iNfreq ;
	iLower_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_iValue[0] ;
	iUpper_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue +iLower_f -1 ;
	
	//`B֐̍쐬
	if(this->m_iFlgtf == 0){	// Calculation
		for(int i = 0 ; i < this->locaMic.ha_numLocation ; i++){
			m_psSepMat->micary_tdTFDB.micary_locaMic.ha_idLocation[i] = locaMic.ha_idLocation[i] ;
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[i].ha_fX = locaMic.micary_Location[i].ha_fX ;
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[i].ha_fY = locaMic.micary_Location[i].ha_fY ;
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[i].ha_fZ = locaMic.micary_Location[i].ha_fZ ;
			// add 2009.7.9
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[i].ha_idPos = locaMic.micary_Location[i].ha_idPos ;
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[i].ha_fPhi = locaMic.micary_Location[i].ha_fPhi ;
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[i].ha_fTheta = locaMic.micary_Location[i].ha_fTheta ;
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[i].micary_LocationFlag = locaMic.micary_Location[i].micary_LocationFlag ;
		}
		// TCY̊mF
		iNch = m_psSepMat->micary_tdTFDB.micary_locaMic.ha_numLocation ;		
		/*
		if(m_bFlgFixedNoise){
			//oĂ΂̂܂
			//oĂȂSourceWFixedNoiseǉ
			if(!this->CheckFixedNoise_noise(m_psSepMat->micary_tdTFDB.micary_locaSource)){
				MakeFixedNoise_Pos(m_psSepMat->micary_tdTFDB.micary_locaSource) ;
			}
		}
		*/
		iNpos = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ;
		iNfreq = iUpper_f - iLower_f +1 ;
		if(m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_numValue != iNch * iNpos * iNfreq)
		{
			//ComplexArrayFree(&m_psSepMat->micary_tdTFDB.micary_cmpaTF) ;	
			ComplexArrayAlloc_r(&m_psSepMat->micary_tdTFDB.micary_cmpaTF,iNch*iNpos*iNfreq, 
				__MICARY_def_MAX_NFL*(__MICARY_def_MAX_NSRC+1)*__MICARY_def_MAX_NCH) ;

		}		
		if(Cexec ::MakeTFMatrix(iUpper_f,psPrmFrame) == 0){
			m_iFlgtf = 0 ;
			m_bFlgFixedNoise = false ;
			m_iFlgsm = 0 ;
			return 0;
		}
		// Conj 
		for(int i = 0 ; i < m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_numValue ; i++){
			m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_cmpValue[i].im = m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_cmpValue[i].im * (-1.0) ; 
		}
	}
	else if(this->m_iFlgtf == 1){	// Read data file
		if(!this->m_psTFDB){	//Initialt[if[^x[X̧ق̓ǂݍ)
			
			m_psTFDB = (MICARY_TFDatabase *)calloc(1,sizeof(MICARY_TFDatabase)) ;	//TFf[^x[X
 			TFDatabaseAlloc(m_psTFDB,0,0,0) ;

			MICARY_TFDBFileHeader * psTFDBHead ;	//TFf[^x[Xwb_
			psTFDBHead = (MICARY_TFDBFileHeader *)calloc(1,sizeof(MICARY_TFDBFileHeader)) ;
	
			HA_String file ; // TF t@C
			file.s = (HA_Char *)calloc(strlen(this->m_file_tf)+1,sizeof(char)) ;
			strcpy(file.s,this->m_file_tf) ;
			file.slen = strlen(this->m_file_tf) ;
	
			if(m_iFormat_TF_CONJ == __FILE_FORMAT_HARK){
				if(cHark.ReadHeader(file.s,psTFDBHead) == 0) //t@C̃wb_擾
				{
					printf("Can not read TF Database.\n") ;
					if(psTFDBHead){
						free(psTFDBHead) ;
						psTFDBHead = NULL ;
					}
					m_iFlgtf = 0 ;
					m_bFlgFixedNoise = false ;
					m_iFlgsm = 0 ;
					if(file.s){
						free(file.s) ;
						file.s = NULL ;
					}
					return 0 ;
				}
			}
		       
		     TFDatabaseAlloc(m_psTFDB,(int)psTFDBHead->lNFreq,(int)psTFDBHead->lNSrc,(int)psTFDBHead->lNMic) ;
		     cout << "NF = " << psTFDBHead->lNFreq << "NSrc = " << psTFDBHead->lNSrc << "NMic = " << psTFDBHead->lNMic << endl ;
		     if(Cexec ::ReadTFMatrix(*psPrmFrame) == 0){
				free(psTFDBHead)  ;
				psTFDBHead = NULL ;
				m_iFlgtf = 0 ;
				m_bFlgFixedNoise = false ;
				m_iFlgsm = 0 ;
				if(file.s){
					free(file.s) ;
					file.s = NULL ;
				}
				return 0;
			}
			iNch = psTFDBHead->lNMic ;
		      free(psTFDBHead)  ;
			psTFDBHead = NULL ;
			if(file.s){
				free(file.s) ;
				file.s = NULL ;
			}
			
		}
		
		// Initialize
		iNfreq = iUpper_f - iLower_f +1 ;
		iNpos = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ;
		// not Init frame 
		iNch = m_psTFDB->micary_locaMic.ha_numLocation ;
		 
		// TCYႦΐ؂Ȃ
		if(m_psSepMat->micary_tdTFDB.micary_locaMic.ha_numLocation != iNch){
			LocationArrayFree(&m_psSepMat->micary_tdTFDB.micary_locaMic) ;   
			LocationArrayAlloc(&m_psSepMat->micary_tdTFDB.micary_locaMic,iNch) ;
		}
		int ii ;
		for(ii = 0 ; ii < iNch ; ii++){
			m_psSepMat->micary_tdTFDB.micary_locaMic.ha_idLocation[ii] = m_psTFDB->micary_locaMic.ha_idLocation[ii] ;
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].ha_fPhi = m_psTFDB->micary_locaMic.micary_Location[ii].ha_fPhi ;
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].ha_fTheta = m_psTFDB->micary_locaMic.micary_Location[ii].ha_fTheta ;
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].ha_fX = m_psTFDB->micary_locaMic.micary_Location[ii].ha_fX ;
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].ha_fY = m_psTFDB->micary_locaMic.micary_Location[ii].ha_fY ;
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].ha_fZ = m_psTFDB->micary_locaMic.micary_Location[ii].ha_fZ ;
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].ha_idDir = m_psTFDB->micary_locaMic.micary_Location[ii].ha_idDir ;
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].ha_idPos = m_psTFDB->micary_locaMic.micary_Location[ii].ha_idPos ;
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].micary_LocationFlag = m_psTFDB->micary_locaMic.micary_Location[ii].micary_LocationFlag ;
		}

		if(m_psSepMat->micary_tdTFDB.micary_iaFreqLine.ha_numValue != iNfreq){
			IntArrayFree(&m_psSepMat->micary_tdTFDB.micary_iaFreqLine) ;   
			IntArrayAlloc(&m_psSepMat->micary_tdTFDB.micary_iaFreqLine,iNfreq) ;
		}
		
		for(ii = 0 ; ii < iNfreq ; ii++){
		  if(ii < m_psTFDB->micary_iaFreqLine.ha_numValue){
			m_psSepMat->micary_tdTFDB.micary_iaFreqLine.ha_iValue[ii] = m_psTFDB->micary_iaFreqLine.ha_iValue[ii] ;
		  }
		  else{
		    m_psSepMat->micary_tdTFDB.micary_iaFreqLine.ha_iValue[ii] = iNfreq ;
		  }
		    
		}

		if(m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_numValue != iNpos *iNch * iNfreq ){
		  //ComplexArrayFree(&m_psSepMat->micary_tdTFDB.micary_cmpaTF) ;
		  ComplexArrayAlloc_r(&m_psSepMat->micary_tdTFDB.micary_cmpaTF,iNch*iNfreq*iNpos,
		  					__MICARY_def_MAX_NFL*(__MICARY_def_MAX_NSRC+1)*__MICARY_def_MAX_NCH) ;                 		
		}
		
		//Kvȏo
		if(Cexec::GetNearestTFMatrixFromDB(&m_psSepMat->micary_tdTFDB,m_psTFDB) == 0){
				m_iFlgtf = 0 ;
				m_bFlgFixedNoise = false ;
				m_iFlgsm = 0 ;
				return 0;
		}

	}

	return 1 ;
}
// -------------------------------------------------------------------------------------
// 2012/01/13 added
// TFf[^x[XAs񏉊lf[^x[X̃t@Cǂݍ
int Cexec :: LoadInitialTFMatrix(MICARY_PrmFrame *psPrmFrame, int iSrcIndex)
{
	int iLower_f,iUpper_f,iNch,iNpos,iNfreq ;
	iLower_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_iValue[0] ;
	iUpper_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue +iLower_f -1 ;
	
	//`B֐̍쐬
	if(this->m_iFlgtf == 0){	// Calculation
		// TCY̊mF
		iNch = m_psSepMat->micary_tdTFDB.micary_locaMic.ha_numLocation ;		
		iNpos = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ;
		iNfreq = iUpper_f - iLower_f +1 ;
		if(Cexec ::MakeTFMatrix2(psPrmFrame, iSrcIndex) == 0){
			m_iFlgtf = 0 ;
			m_bFlgFixedNoise = false ;
			m_iFlgsm = 0 ;
			return 0;
		}
		// Conj
		int idmic, idfreq, index;
		for(idfreq = 0 ; idfreq < iNfreq ; idfreq++){
			for(idmic = 0 ; idmic < iNch ; idmic ++){
				index = idfreq*iNch*iNpos + iSrcIndex*iNch + idmic;
			  	m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_cmpValue[index].im = m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_cmpValue[index].im * (-1.0) ; 
			}
		}
		
	}
	else if(this->m_iFlgtf == 1){	// Read data file

		// Initialize
		iNfreq = iUpper_f - iLower_f +1 ;
		iNpos = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ;
		// not Init frame 
		iNch = m_psTFDB->micary_locaMic.ha_numLocation ;
		
		//Kvȏo
		if(Cexec::GetNearestTFMatrixFromDB2(&m_psSepMat->micary_tdTFDB,m_psTFDB, iSrcIndex) == 0){
				m_iFlgtf = 0 ;
				m_bFlgFixedNoise = false ;
				m_iFlgsm = 0 ;
				return 0;
		}

	}

	return 1 ;
}
// -------------------------------------------------------------------------------------
int Cexec :: InitSepMatrix(MICARY_PrmFrame *psPrmFrame)
{

	//s̏
	if(this->m_iFlgsm == 0){	// Calculation
	 
		if(MakeWMatGHDSS(m_psSepMat->micary_tdTFDB,&m_psSepMat->micary_smSeparation) == 0){
			std ::cout << "Fail to initialize sepatation matrix." << std::endl ;
			m_iFlgtf = 0 ;
			m_bFlgFixedNoise = false ;
			m_iFlgsm = 0 ;
			return 0 ;
		}
		// TFt@CconjŒ`
	
	}
	else if(this->m_iFlgsm == 1){	// Read data file
		if(!m_psSMDB){	//Initial frame process
			
			MICARY_TFDBFileHeader * psSMDBHead ;	//f[^x[Xwb_
			psSMDBHead = (MICARY_TFDBFileHeader *)calloc(1,sizeof(MICARY_TFDBFileHeader)) ;
	
			HA_String file ; // t@C
			file.s = (HA_Char *)calloc(strlen(this->m_file_sm)+1,sizeof(char)) ;
			strcpy(file.s,this->m_file_sm) ;
			file.slen = strlen(this->m_file_sm) ;
	
			if(m_iFormat_InitW == __FILE_FORMAT_HARK){
				if(cHark.ReadHeader(file.s,psSMDBHead) == 0) //t@C̃wb_擾
				{
					printf("Can not read Separation matrix Database.\n") ;
					if(psSMDBHead){
						free(psSMDBHead) ;
						psSMDBHead = NULL ;
					}
					m_iFlgtf = 0 ;
					m_bFlgFixedNoise = false ;
					m_iFlgsm = 0 ;
					if(file.s){
						free(file.s) ;
						file.s = NULL ;
					}
					return 0 ;
				}

			}
			m_psSMDB = (MICARY_SMDatabase *)calloc(1,sizeof(MICARY_SMDatabase)) ;	//sۑl
			SMDatabaseAlloc(m_psSMDB,0,0,0) ;

			SMDatabaseAlloc(m_psSMDB,(int)psSMDBHead->lNFreq,(int)psSMDBHead->lNSrc,(int)psSMDBHead->lNMic) ;
			
			free(psSMDBHead)  ;
			psSMDBHead = NULL ;

			// Separation matrixf[^̓ǂݍ
			if(Cexec ::ReadSepMatrix(*psPrmFrame) == 0){
				m_iFlgtf = 0 ;
				m_bFlgFixedNoise = false ;
				m_iFlgsm = 0 ;
				if(file.s){
					free(file.s) ;
					file.s = NULL ;
				}
				return 0;
			}
			if(file.s){
				free(file.s) ;
				file.s = NULL ;
			}
		} //End initial frame process
		
		// ԋ߂Wo
		int iLower_f,iUpper_f,iNch,iNpos,iNfreq ;
		iLower_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_iValue[0] ;
		iUpper_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue +iLower_f -1 ;
		
		iNch = m_psSepMat->micary_tdTFDB.micary_locaMic.ha_numLocation ;
       	iNfreq = iUpper_f - iLower_f +1 ;
       	iNpos = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ;
	
       	if(m_psSepMat->micary_smSeparation.micary_cmpaSeparation.ha_numValue != iNpos *iNch * iNfreq ){
	 		//ComplexArrayFree(&m_psSepMat->micary_smSeparation.micary_cmpaSeparation) ;	// 2012/01/19 deleted
	 	  	ComplexArrayAlloc_r(&m_psSepMat->micary_smSeparation.micary_cmpaSeparation,iNch*iNfreq*iNpos,
	 	  		__MICARY_def_MAX_NCH*(__MICARY_def_MAX_NSRC+1)*__MICARY_def_MAX_NFL) ;
		  	m_psSepMat->micary_smSeparation.ha_numInput = iNch ;
		  	m_psSepMat->micary_smSeparation.ha_numOutput= iNpos ;
		}
	//2010.09.29 assign freq line index
	for(int i = 0 ; i < m_psSMDB->micary_smSeparation.micary_iaFreqLinex.ha_numValue ; i++){
	  m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_iValue[i] =  m_psSMDB->micary_smSeparation.micary_iaFreqLinex.ha_iValue[i]  ;
	}
		
		//Kvȏo
		if(Cexec::GetNearestSepMatrixFromDB(m_psSepMat,m_psSMDB) == 0){
			m_iFlgtf = 0 ;
			m_bFlgFixedNoise = false ;
			m_iFlgsm = 0 ;
			return 0;
		}
		
	}
	return 1;
}
// ---------------------------------------------------------------------
// 2012/01/13 added
// CfbNXŕs񏉊lǂݍ
int Cexec :: LoadInitialSepMatrix(MICARY_PrmFrame *psPrmFrame, int iSrcIndex)
{
	//s̏
	if(this->m_iFlgsm == 0){	// Calculation
		if(MakeWMatGHDSS2(m_psSepMat->micary_tdTFDB, &m_psSepMat->micary_smSeparation, iSrcIndex) == 0){
			std ::cout << "Fail to initialize sepatation matrix." << std::endl ;
			m_iFlgtf = 0 ;
			m_bFlgFixedNoise = false ;
			m_iFlgsm = 0 ;
			return 0 ;
		}
		// TFt@CconjŒ`
	
	}
	else if(this->m_iFlgsm == 1){	// Read data file
		// ԋ߂Wo
		int iLower_f,iUpper_f,iNch,iNpos,iNfreq ;
		iLower_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_iValue[0] ;
		iUpper_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue +iLower_f -1 ;
		
		iNch = m_psSepMat->micary_tdTFDB.micary_locaMic.ha_numLocation ;
       	iNfreq = iUpper_f - iLower_f +1 ;
       	iNpos = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ;
		
		//Kvȏo
		if(Cexec::GetNearestSepMatrixFromDB2(m_psSepMat,m_psSMDB,iSrcIndex) == 0){
			m_iFlgtf = 0 ;
			m_bFlgFixedNoise = false ;
			m_iFlgsm = 0 ;
			return 0;
		}
		
	}
	return 1;
}
// ---------------------------------------------------------------------
// Wz̑
int Cexec :: AssignSrcPos(MICARY_LocationArray locaSrcPos)
{
	
	int n ;
	if(!&locaSrcPos){
		return 0 ;
	}

	if(this->m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation != locaSrcPos.ha_numLocation)
	{
	  //LocationArrayFree(&m_psSepMat->micary_tdTFDB.micary_locaSource) ;	
	  LocationArrayAlloc_r(&m_psSepMat->micary_tdTFDB.micary_locaSource,locaSrcPos.ha_numLocation,
	  						__MICARY_def_MAX_NSRC+1) ;
	}

	for(n = 0 ; n < locaSrcPos.ha_numLocation ; n ++){
		m_psSepMat->micary_tdTFDB.micary_locaSource.ha_idLocation[n] = locaSrcPos.ha_idLocation[n] ;
		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_fX = locaSrcPos.micary_Location[n].ha_fX ;
		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_fY = locaSrcPos.micary_Location[n].ha_fY ;
		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_fZ = locaSrcPos.micary_Location[n].ha_fZ ;
		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_idPos = locaSrcPos.micary_Location[n].ha_idPos ;

		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].micary_LocationFlag = locaSrcPos.micary_Location[n].micary_LocationFlag ;
		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_fTheta = locaSrcPos.micary_Location[n].ha_fTheta ;
		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_fPhi = locaSrcPos.micary_Location[n].ha_fPhi ;
		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_idDir = 0;
	}

	return 1 ;
}

// ---------------------------------------------------------------------------
/*
// FixedNoise_noiseTF̍쐬pW̍쐬
int Cexec::MakeFixedNoise_Pos(MICARY_LocationArray &locaSrc)
{
	int i ;	
	MICARY_LocationArray locaSrcTmp ; //,locaSrcFixedNoise;
	int iNpos = locaSrc.ha_numLocation ;
	
	//FixedNoiseȊOobNAbv
	LocationArrayAlloc(&locaSrcTmp,0) ;
	LocationArrayAlloc(&locaSrcTmp,iNpos) ;
	for(i = 0 ; i < iNpos ; i++){
		locaSrcTmp.ha_idLocation[i] = locaSrc.ha_idLocation[i] ;
		locaSrcTmp.micary_Location[i].ha_fX = locaSrc.micary_Location[i].ha_fX ;
		locaSrcTmp.micary_Location[i].ha_fY = locaSrc.micary_Location[i].ha_fY ;
		locaSrcTmp.micary_Location[i].ha_fZ = locaSrc.micary_Location[i].ha_fZ ;
		locaSrcTmp.micary_Location[i].ha_idPos = locaSrc.micary_Location[i].ha_idPos ;
		locaSrcTmp.micary_Location[i].ha_fPhi = locaSrc.micary_Location[i].ha_fPhi ;
		locaSrcTmp.micary_Location[i].ha_fTheta = locaSrc.micary_Location[i].ha_fTheta ;
		locaSrcTmp.micary_Location[i].micary_LocationFlag = locaSrc.micary_Location[i].micary_LocationFlag ;
	}
	
	LocationArrayFree(&locaSrc) ;
	LocationArrayAlloc(&locaSrc,iNpos+1) ;
	
	for(i = 0 ; i < iNpos ; i++){
		locaSrc.ha_idLocation[i] = locaSrcTmp.ha_idLocation[i] ;
		locaSrc.micary_Location[i].ha_fX = locaSrcTmp.micary_Location[i].ha_fX ;
		locaSrc.micary_Location[i].ha_fY = locaSrcTmp.micary_Location[i].ha_fY ;
		locaSrc.micary_Location[i].ha_fZ = locaSrcTmp.micary_Location[i].ha_fZ ;
		locaSrc.micary_Location[i].ha_idPos = locaSrcTmp.micary_Location[i].ha_idPos ;
		locaSrc.micary_Location[i].ha_fPhi = locaSrcTmp.micary_Location[i].ha_fPhi ;
		locaSrc.micary_Location[i].ha_fTheta = locaSrcTmp.micary_Location[i].ha_fTheta ;
		locaSrc.micary_Location[i].micary_LocationFlag = locaSrcTmp.micary_Location[i].micary_LocationFlag ;	
	}
	
	locaSrc.micary_Location[iNpos].ha_fX = locaSrcFixedNoise.micary_Location[0].ha_fX ;
	locaSrc.micary_Location[iNpos].ha_fY = locaSrcFixedNoise.micary_Location[0].ha_fY ;
	locaSrc.micary_Location[iNpos].ha_fZ = locaSrcFixedNoise.micary_Location[0].ha_fZ ;
	locaSrc.micary_Location[iNpos].ha_fTheta = locaSrcFixedNoise.micary_Location[0].ha_fTheta ;
	locaSrc.micary_Location[iNpos].ha_fPhi = locaSrcFixedNoise.micary_Location[0].ha_fPhi ;
	locaSrc.micary_Location[iNpos].micary_LocationFlag = locaSrcFixedNoise.micary_Location[0].micary_LocationFlag ;
	// FixedNoise sourceID = -1
	locaSrc.ha_idLocation[iNpos] = -1 ;
	locaSrc.micary_Location[iNpos].ha_idPos = -1 ;
	
	LocationArrayFree(&locaSrcTmp) ;
	
	return 1 ;
}
*/
// FixedNoise_noiseTF̍쐬pW̍쐬
int Cexec::MakeFixedNoise_Pos(MICARY_LocationArray &locaSrc)
{
	int iNpos = locaSrc.ha_numLocation ;
	
	if (locaSrc.ha_numLocation+1 > locaSrc.ha_numReserved){
		// ۂɂ͂̃[gʂ邱Ƃ͂قڂȂ
		int i ;	
		MICARY_LocationArray locaSrcTmp ; //,locaSrcFixedNoise;
	
		//FixedNoiseȊOobNAbv
		LocationArrayAlloc(&locaSrcTmp,0) ;
		LocationArrayAlloc(&locaSrcTmp,iNpos) ;
		for(i = 0 ; i < iNpos ; i++){
			locaSrcTmp.ha_idLocation[i] = locaSrc.ha_idLocation[i] ;
			locaSrcTmp.micary_Location[i].ha_fX = locaSrc.micary_Location[i].ha_fX ;
			locaSrcTmp.micary_Location[i].ha_fY = locaSrc.micary_Location[i].ha_fY ;
			locaSrcTmp.micary_Location[i].ha_fZ = locaSrc.micary_Location[i].ha_fZ ;
			locaSrcTmp.micary_Location[i].ha_idPos = locaSrc.micary_Location[i].ha_idPos ;
			locaSrcTmp.micary_Location[i].ha_fPhi = locaSrc.micary_Location[i].ha_fPhi ;
			locaSrcTmp.micary_Location[i].ha_fTheta = locaSrc.micary_Location[i].ha_fTheta ;
			locaSrcTmp.micary_Location[i].micary_LocationFlag = locaSrc.micary_Location[i].micary_LocationFlag ;
		}
	
		LocationArrayAlloc_r(&locaSrc, iNpos+1, __MICARY_def_MAX_NSRC+1) ;
	
		for(i = 0 ; i < iNpos ; i++){
			locaSrc.ha_idLocation[i] = locaSrcTmp.ha_idLocation[i] ;
			locaSrc.micary_Location[i].ha_fX = locaSrcTmp.micary_Location[i].ha_fX ;
			locaSrc.micary_Location[i].ha_fY = locaSrcTmp.micary_Location[i].ha_fY ;
			locaSrc.micary_Location[i].ha_fZ = locaSrcTmp.micary_Location[i].ha_fZ ;
			locaSrc.micary_Location[i].ha_idPos = locaSrcTmp.micary_Location[i].ha_idPos ;
			locaSrc.micary_Location[i].ha_fPhi = locaSrcTmp.micary_Location[i].ha_fPhi ;
			locaSrc.micary_Location[i].ha_fTheta = locaSrcTmp.micary_Location[i].ha_fTheta ;
			locaSrc.micary_Location[i].micary_LocationFlag = locaSrcTmp.micary_Location[i].micary_LocationFlag ;	
		}

		LocationArrayFree(&locaSrcTmp) ;
	}
	else {
		locaSrc.ha_numLocation += 1;
	}
	
	locaSrc.micary_Location[iNpos].ha_fX = locaSrcFixedNoise.micary_Location[0].ha_fX ;
	locaSrc.micary_Location[iNpos].ha_fY = locaSrcFixedNoise.micary_Location[0].ha_fY ;
	locaSrc.micary_Location[iNpos].ha_fZ = locaSrcFixedNoise.micary_Location[0].ha_fZ ;
	locaSrc.micary_Location[iNpos].ha_fTheta = locaSrcFixedNoise.micary_Location[0].ha_fTheta ;
	locaSrc.micary_Location[iNpos].ha_fPhi = locaSrcFixedNoise.micary_Location[0].ha_fPhi ;
	locaSrc.micary_Location[iNpos].micary_LocationFlag = locaSrcFixedNoise.micary_Location[0].micary_LocationFlag ;
	// FixedNoise sourceID = -1
	locaSrc.ha_idLocation[iNpos] = -1 ;
	locaSrc.micary_Location[iNpos].ha_idPos = -1 ;
		
	return 1 ;
}
// ---------------------------------------------------------------------------
//FixedNoise noisẽ`FbN
/*
bool Cexec::CheckFixedNoise_noise(MICARY_LocationArray locaSrc)
{
	
	bool bFixedNoise_Src = false ;
	int i ,iNpos;
	MICARY_FloatArray laSrc,laFixedNoise ;
	float ha_fNorm,ha_fP ;

	iNpos = locaSrc.ha_numLocation ;
	for(i = 0 ; i < iNpos ; i++){
		if(locaSrc.micary_Location[i].ha_fZ < 0){ // Z<0łFixedNoise_noise
			bFixedNoise_Src = true ;
		}
		  
		else{
			FloatArrayAlloc(&laSrc,0) ;
		   	FloatArrayAlloc(&laFixedNoise,0);

		  	FloatArrayAlloc(&laSrc,3) ;
		  	FloatArrayAlloc(&laFixedNoise,3) ;

		  	ha_fNorm = (float)pow((double)locaSrc.micary_Location[i].ha_fX,(double)2.0) + pow((double)locaSrc.micary_Location[i].ha_fY,(double)2.0)+pow((double)locaSrc.micary_Location[i].ha_fZ,(double)2.0) ;
		  	ha_fNorm = (float)pow((double)ha_fNorm,(double)0.5) ;
	
		  	laSrc.ha_fValue[0] = locaSrc.micary_Location[i].ha_fX / ha_fNorm ;
			laSrc.ha_fValue[1] = locaSrc.micary_Location[i].ha_fY / ha_fNorm ;
			laSrc.ha_fValue[2] = locaSrc.micary_Location[i].ha_fZ / ha_fNorm ;

			ha_fNorm = (float)(pow((double)locaSrcFixedNoise.micary_Location->ha_fX,(double)2.0) + pow((double)locaSrcFixedNoise.micary_Location->ha_fY,(double)2.0)+pow((double)locaSrcFixedNoise.micary_Location->ha_fZ,(double)2.0)) ;
			ha_fNorm = (float)(pow((double)ha_fNorm,(double)0.5)) ;
	
		  	laFixedNoise.ha_fValue[0] = locaSrcFixedNoise.micary_Location->ha_fX / ha_fNorm ;
			laFixedNoise.ha_fValue[1] = locaSrcFixedNoise.micary_Location->ha_fY / ha_fNorm ;
			laFixedNoise.ha_fValue[2] = locaSrcFixedNoise.micary_Location->ha_fZ / ha_fNorm ;
		
			ha_fP = _InnerProductVec(laSrc,laFixedNoise) ;
			
			if(fabs(ha_fP) > 0.999){
		  		locaSrc.ha_idLocation[i]= -1 ;
		  		locaSrc.micary_Location[i].ha_idPos = -1 ;
		  		bFixedNoise_Src = true ;
			}
		 	FloatArrayFree(&laSrc) ;
		  	FloatArrayFree(&laFixedNoise) ;	
		}
	}
	return bFixedNoise_Src ;
	
}
*/
//FixedNoise noisẽ`FbN
bool Cexec::CheckFixedNoise_noise(MICARY_LocationArray locaSrc)
{
	
	bool bFixedNoise_Src = false ;
	int i ,iNpos;
	float fSrc[3], fFixedNoise[3];
	float ha_fNorm,ha_fP ;

	// fixednoisȅ
	ha_fNorm = (float)(pow((double)locaSrcFixedNoise.micary_Location->ha_fX,(double)2.0) + pow((double)locaSrcFixedNoise.micary_Location->ha_fY,(double)2.0)+pow((double)locaSrcFixedNoise.micary_Location->ha_fZ,(double)2.0)) ;
	ha_fNorm = (float)(pow((double)ha_fNorm,(double)0.5)) ;
	
	fFixedNoise[0] = locaSrcFixedNoise.micary_Location->ha_fX / ha_fNorm ;
	fFixedNoise[1] = locaSrcFixedNoise.micary_Location->ha_fY / ha_fNorm ;
	fFixedNoise[2] = locaSrcFixedNoise.micary_Location->ha_fZ / ha_fNorm ;
		

	iNpos = locaSrc.ha_numLocation ;
	for(i = 0 ; i < iNpos ; i++){
		if(locaSrc.micary_Location[i].ha_fZ < 0){ // Z<0łFixedNoise_noise
			bFixedNoise_Src = true ;
		}
		else{
		  	ha_fNorm = (float)pow((double)locaSrc.micary_Location[i].ha_fX,(double)2.0) + pow((double)locaSrc.micary_Location[i].ha_fY,(double)2.0)+pow((double)locaSrc.micary_Location[i].ha_fZ,(double)2.0) ;
		  	ha_fNorm = (float)pow((double)ha_fNorm,(double)0.5) ;
	
		  	fSrc[0] = locaSrc.micary_Location[i].ha_fX / ha_fNorm ;
			fSrc[1] = locaSrc.micary_Location[i].ha_fY / ha_fNorm ;
			fSrc[2] = locaSrc.micary_Location[i].ha_fZ / ha_fNorm ;

			ha_fP = _InnerProductVec(fSrc,fFixedNoise) ;
			
			if(fabs(ha_fP) > 0.999){
		  		locaSrc.ha_idLocation[i]= -1 ;
		  		locaSrc.micary_Location[i].ha_idPos = -1 ;
		  		bFixedNoise_Src = true ;
			}
		}
	}
	return bFixedNoise_Src ;
	
}

// ------------------------------------------------------------------
// sf[^x[Xt@C̓ǂݍ
int Cexec::ReadSepMatrix(MICARY_PrmFrame psPrmFrame)
{
	HA_String SMfile ;
	SMfile.s = (HA_Char *)calloc(strlen(this->m_file_sm)+1,sizeof(char)) ;
	strcpy(SMfile.s,this->m_file_sm) ;
	SMfile.slen = strlen(this->m_file_sm) ;

	MICARY_TFDBFileHeader * psSMDBHead ;	//TFf[^x[Xwb_(ʃtH[}bg)
	psSMDBHead = (MICARY_TFDBFileHeader *)calloc(1,sizeof(MICARY_TFDBFileHeader)) ;
	
	if(m_iFormat_InitW == __FILE_FORMAT_HARK){
		if(!cHark.ReadInfo(SMfile.s,psSMDBHead,
							&m_psSMDB->micary_locaSource ,
							&m_psSMDB->micary_locaMic,
							&m_psSMDB->micary_smSeparation.micary_iaFreqLinex)){
			printf("Can not read SM Database.\n") ;
			if(psSMDBHead){
				free(psSMDBHead) ;
				psSMDBHead = NULL ;
			}
			if(SMfile.s){
				free(SMfile.s) ;
				SMfile.s = NULL ;
			}

			return 0 ;
		}
		
		if(!cHark.ReadData(SMfile.s,
			&m_psSMDB->micary_smSeparation.micary_cmpaSeparation)){
				printf("Can not read SM Database.\n") ;
			if(psSMDBHead){
				free(psSMDBHead) ;
				psSMDBHead = NULL ;
			}
			if(SMfile.s){
				free(SMfile.s) ;
				SMfile.s = NULL ;
			}

			return 0 ;
		}
	}
	
	if(psSMDBHead){
		free(psSMDBHead) ;
		psSMDBHead = NULL ;
	}
	if(SMfile.s){
		free(SMfile.s) ;
		SMfile.s = NULL ;
	}
	
	return 1 ;

}

// ----------------------------------------------------------------
bool Cexec::Initialize_Change(int iNch, int iNpos, int iLower_f, int iUpper_f)
{

 	// ύX̔z̏
	int iSize ;
	iSize = (iUpper_f - iLower_f +1)*iNpos*iNch ;
	if(iSize != m_psSepMat->micary_smSeparation.micary_cmpaSeparation.ha_numValue){	
		// 2012/01/13
		/*
	  	SavedSeparationFree(m_psSepMat) ;
	  	SavedSeparationAlloc(m_psSepMat,iUpper_f-iLower_f+1,iNpos,iNch) ;*/
	  	SavedSeparationAlloc_r(m_psSepMat,iUpper_f-iLower_f+1,iNpos,iNch,
	  							__MICARY_def_MAX_NFL, __MICARY_def_MAX_NSRC+1, __MICARY_def_MAX_NCH) ;
	}

	//gCz̍쐬
	if(iUpper_f-iLower_f >= 1){
		for(int k =0 ; k < iUpper_f-iLower_f +1 ; k++ ){
			m_psSepMat->micary_tdTFDB.micary_iaFreqLine.ha_iValue[k] = iLower_f + k ;
		}
	}

	return true ;
}
// ----------------------------------------------------------------
// xNg̓ς̌vZ
HA_Float Cexec::_InnerProductVec(MICARY_FloatArray micary_faVec1,MICARY_FloatArray micary_faVec2)
{
	int k ;
	HA_Float ha_fRet,tmp ;

	if(micary_faVec1.ha_numValue != micary_faVec2.ha_numValue){
			return -1.0 ;
	}
	
	ha_fRet = 0.0 ;

	for(k = 0 ; k < micary_faVec1.ha_numValue ; k++){
		tmp = micary_faVec1.ha_fValue[k] * micary_faVec2.ha_fValue[k] ;
		ha_fRet = ha_fRet + tmp ;
	}

	return ha_fRet ;

}
// 2011/11/16 added by N.Tanaka
HA_Float Cexec::_InnerProductVec(float fVec1[3],float fVec2[3])
{
	int k ;
	HA_Float ha_fRet;
	ha_fRet = 0.0 ;
	for(k = 0 ; k < 3 ; k++){
		ha_fRet += fVec1[k] * fVec2[k];
	}
	return ha_fRet ;
}
// -----------------------------------------------------------------
// DBԋ߂Index擾
/*
HA_Integer Cexec::GetNearestPos(MICARY_Location micary_loCur, MICARY_LocationArray *micary_laDB)
{
	
	HA_Float ha_fMaxP ,ha_fP,ha_fNorm ;
	HA_Integer ha_iMax ;
	MICARY_FloatArray laCurN , laPosN ;
	int k ;

	ha_fMaxP = -32768.0 ;
	FloatArrayAlloc(&laCurN,0) ;
	FloatArrayAlloc(&laPosN,0) ;

	FloatArrayAlloc(&laCurN,3) ;
	FloatArrayAlloc(&laPosN,3) ;

	// K
	ha_fNorm = pow((double)micary_loCur.ha_fX,2.0) + pow((double)micary_loCur.ha_fY,2.0)+pow((double)micary_loCur.ha_fZ,2.0) ;
	ha_fNorm = pow((double)ha_fNorm,0.5) ;
	
	laCurN.ha_fValue[0] = micary_loCur.ha_fX / ha_fNorm ;
	laCurN.ha_fValue[1] = micary_loCur.ha_fY / ha_fNorm ;
	laCurN.ha_fValue[2] = micary_loCur.ha_fZ / ha_fNorm ;
	
	//DB̊exNgƓςƂ
	for(k = 0 ; k < micary_laDB->ha_numLocation ; k++){
		// K
	  ha_fNorm = pow((double)micary_laDB->micary_Location[k].ha_fX,2.0) + pow((double)micary_laDB->micary_Location[k].ha_fY,2.0)+pow((double)micary_laDB->micary_Location[k].ha_fZ,2.0) ;
	  ha_fNorm = pow((double)ha_fNorm,0.5) ;
		
		laPosN.ha_fValue[0] = micary_laDB->micary_Location[k].ha_fX/ha_fNorm ;
		laPosN.ha_fValue[1] = micary_laDB->micary_Location[k].ha_fY/ha_fNorm ;
		laPosN.ha_fValue[2] = micary_laDB->micary_Location[k].ha_fZ/ha_fNorm ;

		ha_fP = _InnerProductVec(laPosN,laCurN) ;
		if(ha_fP > ha_fMaxP){
			ha_fMaxP = ha_fP ;
			ha_iMax = k ;
		}
	}

	FloatArrayFree(&laCurN) ;
	FloatArrayFree(&laPosN) ;
	// add 2009.7.9
	if(ha_fMaxP <= -32768.0){
		ha_iMax = __INVALID_SRC_ID ;
	}
	return ha_iMax ;
	
}
*/
HA_Integer Cexec::GetNearestPos(MICARY_Location micary_loCur, MICARY_LocationArray *micary_laDB)
{
	
	HA_Float ha_fMaxP ,ha_fP,ha_fNorm ;
	HA_Integer ha_iMax ;
	int k ;
	float fCurN[3], fPosN[3];

	ha_fMaxP = -32768.0 ;

	// K
	ha_fNorm = pow((double)micary_loCur.ha_fX,2.0) + pow((double)micary_loCur.ha_fY,2.0)+pow((double)micary_loCur.ha_fZ,2.0) ;
	ha_fNorm = pow((double)ha_fNorm,0.5) ;
	
	fCurN[0] = micary_loCur.ha_fX / ha_fNorm ;
	fCurN[1] = micary_loCur.ha_fY / ha_fNorm ;
	fCurN[2] = micary_loCur.ha_fZ / ha_fNorm ;
	
	//DB̊exNgƓςƂ
	for(k = 0 ; k < micary_laDB->ha_numLocation ; k++){
		// K
	  ha_fNorm = pow((double)micary_laDB->micary_Location[k].ha_fX,2.0) + pow((double)micary_laDB->micary_Location[k].ha_fY,2.0)+pow((double)micary_laDB->micary_Location[k].ha_fZ,2.0) ;
	  ha_fNorm = pow((double)ha_fNorm,0.5) ;
		
		fPosN[0] = micary_laDB->micary_Location[k].ha_fX/ha_fNorm ;
		fPosN[1] = micary_laDB->micary_Location[k].ha_fY/ha_fNorm ;
		fPosN[2] = micary_laDB->micary_Location[k].ha_fZ/ha_fNorm ;

		ha_fP = _InnerProductVec(fPosN,fCurN) ;
		if(ha_fP > ha_fMaxP){
			ha_fMaxP = ha_fP ;
			ha_iMax = k ;
		}
	}

	// add 2009.7.9
	if(ha_fMaxP <= -32768.0){
		ha_iMax = __INVALID_SRC_ID ;
	}
	return ha_iMax ;
	
}
// --------------------------------------------------------------------------------
// DBf[^̎擾(ł߂Wǂ)
int Cexec::GetNearestTFMatrixFromDB(MICARY_TFDatabase *OutTF,MICARY_TFDatabase *TFDB)
{
	// OutTFɂ͎gCCWz͓Ă
	int i ,m,k,l,Npos,Nmic,NposD,Nfreq,NfreqD,iCount;
	bool bFixedNoise_check ;
	MICARY_Location loctmp ;
	int iDBpos ;

	bFixedNoise_check = false ;
	Npos = OutTF->micary_locaSource.ha_numLocation ;
	NposD = TFDB->micary_locaSource.ha_numLocation ;
	Nmic = TFDB->micary_locaMic.ha_numLocation ;
	NfreqD = TFDB->micary_iaFreqLine.ha_numValue ;
	Nfreq = OutTF->micary_iaFreqLine.ha_numValue ;
	
	if(OutTF->micary_cmpaTF.ha_numValue != Npos*Nmic*Nfreq){
	  cout << "OutTF-size = " << OutTF->micary_cmpaTF.ha_numValue << endl ;
	  cout << "Srcsize = " << Npos << endl ;
	  cout << "freqsize = " << Nfreq << endl ;
	  cout << "micsize = " << Nmic << endl ;
	  cout << "Size error." << endl ;
	  return 0 ;
	}

	iCount = 0 ;

	for(i = 0 ; i < Npos ; i++){
		memcpy(&loctmp,&OutTF->micary_locaSource.micary_Location[i],sizeof(MICARY_Location)) ;
		if(loctmp.ha_fZ < 0 && OutTF->micary_locaSource.ha_idLocation[i] != -1){
				OutTF->micary_locaSource.ha_idLocation[i] = -1 ;	//FixedNoisemCYID = -1
		}
		if(loctmp.ha_fZ < 0 || OutTF->micary_locaSource.ha_idLocation[i] == -1){
			bFixedNoise_check = true ;
		}

		// ԋ߂IDDB擾
		iDBpos = GetNearestPos(loctmp,&TFDB->micary_locaSource) ;
		if(iDBpos != __INVALID_SRC_ID){
		       for(k = 0 ; k < Nfreq ; k++){
				for (l = 0 ; l < NfreqD ; l++){
					if(OutTF->micary_iaFreqLine.ha_iValue[k] == TFDB->micary_iaFreqLine.ha_iValue[l]){
						for(m = 0 ; m < Nmic ; m++){
							// TFzfreq->src->mic̏ɕł
							OutTF->micary_cmpaTF.ha_cmpValue[m+(Npos*Nmic)*k+i*Nmic].re = 
									TFDB->micary_cmpaTF.ha_cmpValue[m+(NposD*Nmic)*l+iDBpos*Nmic].re ;
								OutTF->micary_cmpaTF.ha_cmpValue[m+(Npos*Nmic)*k+i*Nmic].im = 
									TFDB->micary_cmpaTF.ha_cmpValue[m+(NposD*Nmic)*l+iDBpos*Nmic].im ;
						}
						break ;
					}
				}
		       }
		       iCount++ ;
		}
	}

	if(this->m_bFlgFixedNoise){
		if(bFixedNoise_check == false){	//͂Ȃ̂Ƀt@CɂȂ΃G[
		  cout << "Cannot find FixedNoise TF data." << endl ;
			return 0 ;
		}
	}
	if(iCount != Npos){
	  cout << "Cannot find the same number of TF data.";
	  cout << "Detect size = "<<iCount << "/ Source size = "<< Npos <<endl ;
		return 0 ; //Ȃ΃G[
	}
	else {
		return 1 ;
	}

}

// --------------------------------------------------------------------------
// DBf[^̎擾(ł߂Wǂ)
int Cexec::GetNearestTFMatrixFromDB2(MICARY_TFDatabase *OutTF,MICARY_TFDatabase *TFDB, int idsrc)
{
	// OutTFɂ͎gCCWz͓Ă
	int i ,m,k,l,Npos,Nmic,NposD,Nfreq,NfreqD,iCount;
	bool bFixedNoise_check ;
	MICARY_Location loctmp ;
	int iDBpos ;

	bFixedNoise_check = false ;
	Npos = OutTF->micary_locaSource.ha_numLocation ;
	NposD = TFDB->micary_locaSource.ha_numLocation ;
	Nmic = TFDB->micary_locaMic.ha_numLocation ;
	NfreqD = TFDB->micary_iaFreqLine.ha_numValue ;
	Nfreq = OutTF->micary_iaFreqLine.ha_numValue ;
	
	if(OutTF->micary_cmpaTF.ha_numValue != Npos*Nmic*Nfreq){
	  cout << "OutTF-size = " << OutTF->micary_cmpaTF.ha_numValue << endl ;
	  cout << "Srcsize = " << Npos << endl ;
	  cout << "freqsize = " << Nfreq << endl ;
	  cout << "micsize = " << Nmic << endl ;
	  cout << "Size error." << endl ;
	  return 0 ;
	}

	if(OutTF->micary_locaSource.micary_Location[idsrc].ha_fZ < 0 && OutTF->micary_locaSource.ha_idLocation[idsrc] != -1){
		OutTF->micary_locaSource.ha_idLocation[idsrc] = -1 ;	//FixedNoisemCYID = -1
	}
	if(OutTF->micary_locaSource.micary_Location[idsrc].ha_fZ < 0 || OutTF->micary_locaSource.ha_idLocation[idsrc] == -1){
		bFixedNoise_check = true ;
	}

	// ԋ߂IDDB擾
	iDBpos = GetNearestPos(OutTF->micary_locaSource.micary_Location[idsrc],&TFDB->micary_locaSource) ;
	if(iDBpos != __INVALID_SRC_ID){
		for(k = 0 ; k < Nfreq ; k++){
			for (l = 0 ; l < NfreqD ; l++){
				if(OutTF->micary_iaFreqLine.ha_iValue[k] == TFDB->micary_iaFreqLine.ha_iValue[l]){
					for(m = 0 ; m < Nmic ; m++){
						// TFzfreq->src->mic̏ɕł
						OutTF->micary_cmpaTF.ha_cmpValue[m+(Npos*Nmic)*k+idsrc*Nmic].re = 
							TFDB->micary_cmpaTF.ha_cmpValue[m+(NposD*Nmic)*l+iDBpos*Nmic].re ;
						OutTF->micary_cmpaTF.ha_cmpValue[m+(Npos*Nmic)*k+idsrc*Nmic].im = 
							TFDB->micary_cmpaTF.ha_cmpValue[m+(NposD*Nmic)*l+iDBpos*Nmic].im ;
					}
					break ;
				}
			}
		}
	}

	if(this->m_bFlgFixedNoise){
		if(bFixedNoise_check == false){	//͂Ȃ̂Ƀt@CɂȂ΃G[
		  cout << "Cannot find FixedNoise TF data." << endl ;
			return 0 ;
		}
	}

	return 1 ;
}

// --------------------------------------------------------------------------
void Cexec::SetiMicPosShift(int ivalue)
{
	iMicPosShift = ivalue ;
	return ;
}

// --------------------------------------------------------------------------
int Cexec::GetNearestSepMatrixFromDB(MICARY_SavedSeparation *OutSM, MICARY_SMDatabase *SMDB)
{

	int i ,m,k,l,Npos,Nmic,NposD,Nfreq,NfreqD,iCount;
	MICARY_Location loctmp ;
	int iDBpos ;
	bool bFixedNoise_check ;
	bFixedNoise_check = false ;

	Npos = OutSM->micary_tdTFDB.micary_locaSource.ha_numLocation ;
	NposD = SMDB->micary_locaSource.ha_numLocation ;
	Nmic = SMDB->micary_locaMic.ha_numLocation ;
	NfreqD = SMDB->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;
	Nfreq = OutSM->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;

	if(OutSM->micary_smSeparation.micary_cmpaSeparation.ha_numValue != Npos*Nmic*Nfreq){
	  cout << "OutSM-size = " << OutSM->micary_smSeparation.micary_cmpaSeparation.ha_numValue << endl ;
	  cout << "Srcsize = " << Npos << endl ;
	  cout << "freqsize = " << Nfreq << endl ;
	  cout << "micsize = " << Nmic << endl ;
	  cout << "Size error." << endl ;
	return 0 ;
	}

	iCount = 0 ;

	for(i = 0 ; i < Npos ; i++){
		memcpy(&loctmp,&OutSM->micary_tdTFDB.micary_locaSource.micary_Location[i],sizeof(MICARY_Location)) ;
	
		if(loctmp.ha_fZ < 0 && OutSM->micary_tdTFDB.micary_locaSource.ha_idLocation[i] != -1){
				OutSM->micary_tdTFDB.micary_locaSource.ha_idLocation[i] = -1 ;	//FixedNoisemCYID = -1
		}
		if(loctmp.ha_fZ < 0 || OutSM->micary_tdTFDB.micary_locaSource.ha_idLocation[i] == -1){
			bFixedNoise_check = true ;
		}

		// ԋ߂IDDB擾
		iDBpos = GetNearestPos(loctmp,&SMDB->micary_locaSource) ;
		if(iDBpos != __INVALID_SRC_ID){
		       for(k = 0 ; k < Nfreq ; k++){
				for (l = 0 ; l < NfreqD ; l++){
					if(OutSM->micary_tdTFDB.micary_iaFreqLine.ha_iValue[k] == SMDB->micary_smSeparation.micary_iaFreqLinex.ha_iValue[l]){
						
						for(m = 0 ; m < Nmic ; m++){
							
							// SepMatzfreq->mic->src̏ɕł
							OutSM->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[m*Npos+(Npos*Nmic)*k+i].re = 
									SMDB->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[m*NposD+(NposD*Nmic)*l+iDBpos].re ;
							OutSM->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[m*Npos+(Npos*Nmic)*k+i].im = 
									SMDB->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[m*NposD+(NposD*Nmic)*l+iDBpos].im ;

						}
						break ;
					}
				}
		       }
	
		       iCount++ ;
		}
	}

	if(this->m_bFlgFixedNoise){
		if(bFixedNoise_check == false){	//͂Ȃ̂Ƀt@CɂȂ΃G[
		  cout << "Cannot find FixedNoise TF data." << endl ;
			return 0 ;
		}
	}
	if(iCount != Npos){
	  cout << "Cannot find the same number of TF data.";
	  cout << "Detect size = "<<iCount << "/ Source size = "<< Npos <<endl ;
		return 0 ; //Ȃ΃G[
	}
	else {
	 
		return 1 ;
	}
	
}
// ----------------------------------------------------------------------------------
// 2012/01/13 added
int Cexec::GetNearestSepMatrixFromDB2(MICARY_SavedSeparation *OutSM, MICARY_SMDatabase *SMDB, int idsrc)
{
	int i ,m,k,l,Npos,Nmic,NposD,Nfreq,NfreqD,iCount;
	MICARY_Location loctmp ;
	int iDBpos ;
	bool bFixedNoise_check ;
	bFixedNoise_check = false ;

	Npos = OutSM->micary_tdTFDB.micary_locaSource.ha_numLocation ;
	NposD = SMDB->micary_locaSource.ha_numLocation ;
	Nmic = SMDB->micary_locaMic.ha_numLocation ;
	NfreqD = SMDB->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;
	Nfreq = OutSM->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;

	if(OutSM->micary_smSeparation.micary_cmpaSeparation.ha_numValue != Npos*Nmic*Nfreq){
	  cout << "OutSM-size = " << OutSM->micary_smSeparation.micary_cmpaSeparation.ha_numValue << endl ;
	  cout << "Srcsize = " << Npos << endl ;
	  cout << "freqsize = " << Nfreq << endl ;
	  cout << "micsize = " << Nmic << endl ;
	  cout << "Size error." << endl ;
	return 0 ;
	}

	iCount = 0 ;

	if(OutSM->micary_tdTFDB.micary_locaSource.micary_Location[idsrc].ha_fZ < 0 && OutSM->micary_tdTFDB.micary_locaSource.ha_idLocation[i] != -1){
			OutSM->micary_tdTFDB.micary_locaSource.ha_idLocation[i] = -1 ;	//FixedNoisemCYID = -1
	}
	if(OutSM->micary_tdTFDB.micary_locaSource.micary_Location[idsrc].ha_fZ < 0 || OutSM->micary_tdTFDB.micary_locaSource.ha_idLocation[i] == -1){
		bFixedNoise_check = true ;
	}
	// ԋ߂IDDB擾
	iDBpos = GetNearestPos(OutSM->micary_tdTFDB.micary_locaSource.micary_Location[idsrc],&SMDB->micary_locaSource) ;
	if(iDBpos != __INVALID_SRC_ID){
		for(k = 0 ; k < Nfreq ; k++){
			for (l = 0 ; l < NfreqD ; l++){
				if(OutSM->micary_tdTFDB.micary_iaFreqLine.ha_iValue[k] == SMDB->micary_smSeparation.micary_iaFreqLinex.ha_iValue[l]){
					for(m = 0 ; m < Nmic ; m++){
						// SepMatzfreq->mic->src̏ɕł
						OutSM->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[m*Npos+(Npos*Nmic)*k+idsrc].re = 
								SMDB->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[m*NposD+(NposD*Nmic)*l+iDBpos].re ;
						OutSM->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[m*Npos+(Npos*Nmic)*k+idsrc].im = 
								SMDB->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[m*NposD+(NposD*Nmic)*l+iDBpos].im ;
					}
					break ;
				}
			}
		}
		iCount++ ;
	}

	if(this->m_bFlgFixedNoise){
		if(bFixedNoise_check == false){	//͂Ȃ̂Ƀt@CɂȂ΃G[
		  cout << "Cannot find FixedNoise TF data." << endl ;
			return 0 ;
		}
	}
	// changed 2012/02/29
	/*
	if(iCount != Npos){
		cout << "Cannot find the same number of TF data.";
	  cout << "Detect size = "<<iCount << "/ Source size = "<< Npos <<endl ;
		return 0 ; //Ȃ΃G[
	}
	else {
	 
		return 1 ;
	}
	*/
	if(iCount != 1){
		cout << "Cannot find the same number of TF data.";
		return 0 ; //Ȃ΃G[
	}
	else {
	 
		return 1 ;
	}
	
}
// ----------------------------------------------------------------------------------
bool Cexec::SetPrmUpdate(int iUpdateMTF_CONJ, int iUpdateMW, float fThred_angle)
{
	// INITW쐬@@= CALCUpdateMethod->ID
	m_iUpdateMethodTF_CONJ = iUpdateMTF_CONJ ;
	m_iUpdateMethodW = iUpdateMW ;
	m_fThred_angle = fThred_angle ;
	return true;
}
// ----------------------------------------------------------------------------------
int Cexec::InitExportW(const char *cFilename,MICARY_PrmFrame *psPrmFrame)
{
	int ii ;

	if(!m_psSepMat){
		return 0 ;
	}

	int iNLoc = 360 / __SAVED_LOC_THETA_PITCH+1 ;
	int iNmic = m_psSepMat->micary_smSeparation.ha_numInput ;
	int iNfreq = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;
	/*
	UpdateSeparationMatAlloc(m_updateW,iNfreq,iNLoc,iNmic) ;

	//XV̏
	for(ii = 0 ; ii < m_updateW->micary_iaUpdate.ha_numValue; ii++){
		m_updateW->micary_iaUpdate.ha_iValue[ii] = __INVALID_SRC_ID ;
	}

	//}CNʒȗ
	for(ii = 0 ; ii < iNmic ; ii++){
		m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[ii].ha_fX =
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].ha_fX ;
		m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[ii].ha_fY =
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].ha_fY ;
		m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[ii].ha_fZ =
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].ha_fZ ;
		m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[ii].ha_idPos =
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].ha_idPos ;
		m_updateW->micary_dbSeparation.micary_locaMic.ha_idLocation[ii] =
			m_psSepMat->micary_tdTFDB.micary_locaMic.ha_idLocation[ii] ;
	}

	// gC̑
	for(ii = 0 ; ii < iNfreq; ii++){
		m_updateW->micary_dbSeparation.micary_smSeparation.micary_iaFreqLinex.ha_iValue[ii] =
				m_psSepMat->micary_tdTFDB.micary_iaFreqLine.ha_iValue[ii] ;
	}

	//ʒu̍쐬
	HA_Float fX,fY,fZ,fTheta,fPhi,pi ;
	pi = acos(-1.0) ;
	fPhi = 16.7 *pi/180.0;

	for(ii = 0 ; ii < iNLoc ; ii++){
		fTheta = ii*__SAVED_LOC_THETA_PITCH*pi/180.0 ;
		fX = cos(fTheta)*cos(fPhi) ;
		fY = sin(fTheta)*cos(fPhi) ;
		fZ = sin(fPhi) ;

		m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_fX = fX ;
		m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_fY = fY ;
		m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_fZ = fZ ;
	}

	//ID̏
	for(ii = 0 ; ii < iNLoc ; ii++){
	  m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[ii] = __INVALID_SRC_ID ;//ii
	  m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_idPos = __INVALID_SRC_ID ; //ii

	}
*/
	//o̓t@C̐ݒ
	if(cFilename){
		HA_String file ; // TF t@C
		file.s = (HA_Char *)calloc(strlen(cFilename)+1,sizeof(char)) ;
		strcpy(file.s,cFilename) ;
		file.slen = strlen(cFilename) ;
		
		MICARY_SMDatabase  *psDum ;	//_~[f[^
		
		psDum = (MICARY_SMDatabase *)calloc(1,sizeof(MICARY_SMDatabase)) ;
		SMDatabaseAlloc(psDum,0,0,0) ;
		SMDatabaseAlloc(psDum,iNfreq,1,iNmic) ;

		//}CNW
		for(ii = 0 ; ii < iNmic ; ii++){
			psDum->micary_locaMic.ha_idLocation[ii]=
				m_updateW->micary_dbSeparation.micary_locaMic.ha_idLocation[ii] ;
			psDum->micary_locaMic.micary_Location[ii].ha_fX = 
				m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[ii].ha_fX ;
			psDum->micary_locaMic.micary_Location[ii].ha_fY = 
				m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[ii].ha_fY ;
			psDum->micary_locaMic.micary_Location[ii].ha_fZ = 
				m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[ii].ha_fZ ;
			psDum->micary_locaMic.micary_Location[ii].ha_idPos = 
				m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[ii].ha_idPos ;
		}
		//gC
		for(ii = 0 ; ii < iNfreq ; ii++){
			psDum->micary_smSeparation.micary_iaFreqLinex.ha_iValue[ii] = 
				m_updateW->micary_dbSeparation.micary_smSeparation.micary_iaFreqLinex.ha_iValue[ii] ;
		}

		// dummy source
		psDum->micary_locaSource.ha_idLocation[0]=
				m_psSepMat->micary_tdTFDB.micary_locaSource.ha_idLocation[0] ;
		psDum->micary_locaSource.micary_Location[0].ha_fX = 
		               	m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[0].ha_fX ;
		  
		psDum->micary_locaSource.micary_Location[0].ha_fY = 
		                m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[0].ha_fY ;

		psDum->micary_locaSource.micary_Location[0].ha_fZ =
		                  m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[0].ha_fZ ;

		psDum->micary_locaSource.micary_Location[0].ha_idPos = 
				  m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[0].ha_idPos ;

		for(ii = 0 ; ii < psDum->micary_smSeparation.micary_cmpaSeparation.ha_numValue ; ii++){
		  psDum->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[ii].re = 0.0 ;
		  psDum->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[ii].im = 0.0 ;
		}

		if(m_iFormat_ExportW == __FILE_FORMAT_HARK){
		  	MICARY_TFDBFileHeader * psH ;	//f[^x[Xwb_
			psH = (MICARY_TFDBFileHeader *)calloc(1,sizeof(MICARY_TFDBFileHeader)) ;
			psH->lFs = (int)psPrmFrame->ha_fFs ;
			psH->lNFFT = psPrmFrame->ha_numNFFT ;
			psH->lNMic =iNmic ;
		        psH->lNSrc = 1 ;
			psH->lNFreq = iNfreq ;
			psH->iVersion = 1 ;
			psH->iFileType = FORMAT_TYPE_W ;

			if(!cHark.WriteData(file.s,psH,
					    psDum->micary_smSeparation.micary_cmpaSeparation,
					    psDum->micary_smSeparation.micary_iaFreqLinex,
					    psDum->micary_locaSource,
					    psDum->micary_locaMic,
					    "",
					    "",
					    "",
					    "",
					    "",
					    "GHDSS_EXPORT")){
			  if(psH){
			      free(psH) ;
			      psH = NULL ;
			  }
			  if(file.s){
			      free(file.s) ;
			      file.s = NULL ;
			 }
			  SMDatabaseFree(psDum) ;
			  free(psDum) ;
			  psDum = NULL ;
			  cout << "Fail to Export file as HARK format." << endl ;
			  return false ;
		      }
		}

	
		if(file.s){
			free(file.s) ;
			file.s = NULL ;
		}
		SMDatabaseFree(psDum) ;
		
		free(psDum) ;
		psDum = NULL ;
		
	}

	return 1 ;
	
}
// ----------------------------------------------------------------------------------
int Cexec::InitUpdateW()
{
	int ii ;

	if(!m_psSepMat){
		return 0 ;
	}
	int iNLoc = 360 / __SAVED_LOC_THETA_PITCH+1 ;
	int iNmic = m_psSepMat->micary_smSeparation.ha_numInput ;
	int iNfreq = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;

	UpdateSeparationMatAlloc(m_updateW,iNfreq,iNLoc,iNmic) ;

	//XV̏
	for(ii = 0 ; ii < m_updateW->micary_iaUpdate.ha_numValue; ii++){
		m_updateW->micary_iaUpdate.ha_iValue[ii] = __INVALID_SRC_ID ;
	}

	//}CNʒȗ
	for(ii = 0 ; ii < iNmic ; ii++){
		m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[ii].ha_fX =
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].ha_fX ;
		m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[ii].ha_fY =
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].ha_fY ;
		m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[ii].ha_fZ =
			m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].ha_fZ ;
		m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[ii].ha_idPos =
		  m_psSepMat->micary_tdTFDB.micary_locaMic.micary_Location[ii].ha_idPos; 
		m_updateW->micary_dbSeparation.micary_locaMic.ha_idLocation[ii] =
			m_psSepMat->micary_tdTFDB.micary_locaMic.ha_idLocation[ii] ;
	}

	// gC̑
	for(ii = 0 ; ii < iNfreq; ii++){
		m_updateW->micary_dbSeparation.micary_smSeparation.micary_iaFreqLinex.ha_iValue[ii] =
				m_psSepMat->micary_tdTFDB.micary_iaFreqLine.ha_iValue[ii] ;
	}

	//ʒu̍쐬
	HA_Float fX,fY,fZ,fTheta,fPhi,pi ;
	pi = acos(-1.0) ;
	fPhi = 16.7 *pi/180.0;

	for(ii = 0 ; ii < iNLoc ; ii++){
		fTheta = ii*__SAVED_LOC_THETA_PITCH*pi/180.0 ;
		fX = cos(fTheta)*cos(fPhi) ;
		fY = sin(fTheta)*cos(fPhi) ;
		fZ = sin(fPhi) ;

		m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_fX = fX ;
		m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_fY = fY ;
		m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_fZ = fZ ;
	}

	//ID̏
	for(ii = 0 ; ii < iNLoc ; ii++){
		m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[ii] = ii ;//__INVALID_SRC_ID ;
		m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_idPos = ii ;
	}
	return 1 ;
	
}	
// ---------------------------------------------------------------------------------------
// 2012/01/13 added
// TF_CONJẄp
int Cexec::ChangeSrcPos_TF_CONJ_W(MICARY_LocationArray plaSrcPosCur, MICARY_PrmFrame psPrmFrame)
{
	int i, k, j, m,  n ;
	int Npos_ini,Npos_cur,Nmic,Nfreq,iLower_f,iUpper_f ;
	//MICARY_ComplexArray tmp_cmpaTFDB ;
	//MICARY_LocationArray plaSrcPosIni ;
	bool bFound, bRtnPos, bRtnID ;
	int iRtn = 2 ;
	int iInherit_TF_CONJ = 0;
	int iInherit_W = 0;

	Nmic = m_psSepMat->micary_smSeparation.ha_numInput ;
	Npos_ini = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ;
	iLower_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_iValue[0] ;
	iUpper_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue + iLower_f -1 ;
	Nfreq = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;
	Npos_cur = plaSrcPosCur.ha_numLocation ;
	
	// obNAbv
		// ʒu
	//LocationArrayAlloc(&plaSrcPosIni,0) ;
	LocationArrayAlloc_r(&m_plaSrcPosIni,Npos_ini,__MICARY_def_MAX_NSRC+1) ;
	for(n = 0 ; n < Npos_ini ; n++){
	   m_plaSrcPosIni.ha_idLocation[n] = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_idLocation[n] ;
	   m_plaSrcPosIni.micary_Location[n].ha_fX = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_fX ;
	   m_plaSrcPosIni.micary_Location[n].ha_fY = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_fY ;
	   m_plaSrcPosIni.micary_Location[n].ha_fZ = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_fZ ;
	   m_plaSrcPosIni.micary_Location[n].ha_idPos = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_idPos ;
	   m_plaSrcPosIni.micary_Location[n].ha_fTheta = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_fTheta ;
	   m_plaSrcPosIni.micary_Location[n].ha_fPhi = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_fPhi ;
	   m_plaSrcPosIni.micary_Location[n].micary_LocationFlag = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].micary_LocationFlag ;
	}
		// TF_CONJ
	//ComplexArrayAlloc(&tmp_cmpaTFDB,0) ;
	ComplexArrayAlloc_r(&m_tmp_cmpaTFDB,(int)Nfreq*Npos_ini*Nmic, __MICARY_def_MAX_NCH*(__MICARY_def_MAX_NSRC+1)*__MICARY_def_MAX_NFL) ;	
	for (n = 0 ; n < m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_numValue;n++){
		m_tmp_cmpaTFDB.ha_cmpValue[n].re = m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_cmpValue[n].re ;
		m_tmp_cmpaTFDB.ha_cmpValue[n].im = m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_cmpValue[n].im ;
	}

	// z񐔂̐؂Ȃ
	if(Npos_ini != Npos_cur){
		Initialize_Change(Nmic,Npos_cur,iLower_f,iUpper_f) ; // Œg̓NAȂ
		iRtn = 1 ;
	}

	// ̍XV
	Cexec ::AssignSrcPos(plaSrcPosCur) ;

	// TF_CONJ̍XV
	if (m_iUpdateMethodTF_CONJ == __UPDATE_METHOD_ID){
		for (i = 0 ; i < Npos_cur ; i++){
			bFound = false;
			for (j = 0 ; j < Npos_ini ; j++){
				if (plaSrcPosCur.ha_idLocation[i] == m_plaSrcPosIni.ha_idLocation[j]){
					// changed 2012/02/29
					// 1zȂ̂ŁAقȂꍇ̓Rs[Kv
//					if (i != j){	// CfbNXł΃Rs[Kv͂Ȃ
//						// OID͈̓̂p
//						for(k = 0 ; k < Nfreq ; k++){
//							for(m = 0 ; m < Nmic ; m++){
//								m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_cmpValue[m+i*Nmic+k*Nmic*Npos_cur].re 
//									= m_tmp_cmpaTFDB.ha_cmpValue[m+j*Nmic+k*Nmic*Npos_ini].re ;
//		   						m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_cmpValue[m+i*Nmic+k*Nmic*Npos_cur].im 
//									= m_tmp_cmpaTFDB.ha_cmpValue[m+j*Nmic+k*Nmic*Npos_ini].im ;
//							}
//						}
//					}
					if (i != j || // CfbNXł΃Rs[Kv͂Ȃ
						Npos_ini != Npos_cur){	
						// OID͈̓̂p
						for(k = 0 ; k < Nfreq ; k++){
							for(m = 0 ; m < Nmic ; m++){
								m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_cmpValue[m+i*Nmic+k*Nmic*Npos_cur].re 
									= m_tmp_cmpaTFDB.ha_cmpValue[m+j*Nmic+k*Nmic*Npos_ini].re ;
		   						m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_cmpValue[m+i*Nmic+k*Nmic*Npos_cur].im 
									= m_tmp_cmpaTFDB.ha_cmpValue[m+j*Nmic+k*Nmic*Npos_ini].im ;
							}
						}
					}
					iInherit_TF_CONJ++ ;
					bFound = true;
					break;
				}
			}
			if (!bFound){
				// TF_CONJlǂݏo
				LoadInitialTFMatrix(&psPrmFrame, i);
			}
		}
	}
	else {
		Cexec::InitTFMatrix(&psPrmFrame);
	}
	
	// W̍XV
//printf("W:");
	for(i = 0 ; i < Npos_cur ; i++){
		
		// added 2012/02/29
		bRtnID = false;
		bRtnPos = false;
		
		m_iUpdateMethodWex->ha_iValue[i] = m_iUpdateMethodW ;
		
		if(m_iUpdateMethodW == __UPDATE_METHOD_ID 
				|| m_iUpdateMethodW == __UPDATE_METHOD_ID_POS){
			// changed 2012/02/29
			// 1zȂ̂ŁAقȂꍇ̓Rs[Kv
//			// IDx[X̊mF
//			if (i < Npos_ini){
//				if (plaSrcPosCur.ha_idLocation[i] == m_plaSrcPosIni.ha_idLocation[i]){
//					bRtnID = true;	// Rs[KvȂ
//				}
//			}
			if (i < Npos_ini &&
				Npos_ini == Npos_cur){
				if (plaSrcPosCur.ha_idLocation[i] == m_plaSrcPosIni.ha_idLocation[i]){
//printf("10(%d),",i);
					bRtnID = true;	// Rs[KvȂ
				}
			}
			if (!bRtnID){
				bRtnID = _CheckSrcID_W(i) ;
//printf("20(%d),",i);
			}
		}
		// ۂ̍XVID or POS
		if(m_iUpdateMethodW == __UPDATE_METHOD_ID_POS){
			if(bRtnID == false){
//printf("30(%d),",i);
				m_iUpdateMethodWex->ha_iValue[i] = __UPDATE_METHOD_POS ;
			}
			else{
//printf("40(%d),",i);
				m_iUpdateMethodWex->ha_iValue[i] = __UPDATE_METHOD_ID ;
			}
		}
		if(m_iUpdateMethodWex->ha_iValue[i] == __UPDATE_METHOD_POS){
			// POSx[X̊mF
//printf("50(%d),",i);
			bRtnPos = _CheckSrcPos_W(i) ;
			if(!bRtnPos){
//printf("60(%d),",i);
			  if(m_iUpdateMethodW == __UPDATE_METHOD_ID_POS){
			    m_iUpdateMethodWex->ha_iValue[i] = __UPDATE_METHOD_ID ;
			  }
			}
			
		}
		
		if(bRtnID || bRtnPos){
//printf("70(%d)\n",i);
			iInherit_W++ ;
		}
		else {
			// SepMat̏l
//printf("80(%d)\n",i);
			LoadInitialSepMatrix(&psPrmFrame, i);
		}
	}

	if(iRtn == 1){	// ύX
		return 1 ;
	}
	else{
		if (iInherit_TF_CONJ != Npos_ini) { // ʒuύXA͕s
			return 2 ;
		}
		else {
			return 0 ;
		}
	}
}
// ---------------------------------------------------------------------------------------
int Cexec::ChangeSrcPos_TF_CONJ(MICARY_LocationArray plaSrcPosCur, MICARY_PrmFrame psPrmFrame)
{
	int n ;
	int Npos_ini,Npos_cur,Nmic,Nfreq,iLower_f,iUpper_f ;
	MICARY_ComplexArray tmp_cmpaTFDB ;
	MICARY_LocationArray plaSrcPosIni ;
	bool bRtn ;
	int iRtn = 2 ;

	Nmic = m_psSepMat->micary_smSeparation.ha_numInput ;
	Npos_ini = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ;
	iLower_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_iValue[0] ;
	iUpper_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue + iLower_f -1 ;
	Nfreq = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;
	Npos_cur = plaSrcPosCur.ha_numLocation ;
	
	// obNAbv
	LocationArrayAlloc(&plaSrcPosIni,0) ;
	LocationArrayAlloc(&plaSrcPosIni,Npos_ini) ;
	for(n = 0 ; n < Npos_ini ; n++){
	   plaSrcPosIni.ha_idLocation[n] = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_idLocation[n] ;
	   plaSrcPosIni.micary_Location[n].ha_fX = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_fX ;
	   plaSrcPosIni.micary_Location[n].ha_fY = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_fY ;
	   plaSrcPosIni.micary_Location[n].ha_fZ = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_fZ ;
	   plaSrcPosIni.micary_Location[n].ha_idPos = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_idPos ;
	   plaSrcPosIni.micary_Location[n].ha_fTheta = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_fTheta ;
	   plaSrcPosIni.micary_Location[n].ha_fPhi = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].ha_fPhi ;
	   plaSrcPosIni.micary_Location[n].micary_LocationFlag = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[n].micary_LocationFlag ;
	}

	ComplexArrayAlloc(&tmp_cmpaTFDB,0) ;
    	ComplexArrayAlloc(&tmp_cmpaTFDB,(int)Nfreq*Npos_ini*Nmic) ;	
    	for (n = 0 ; n < m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_numValue;n++){
		tmp_cmpaTFDB.ha_cmpValue[n].re = m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_cmpValue[n].re ;
		tmp_cmpaTFDB.ha_cmpValue[n].im = m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_cmpValue[n].im ;
	}

	//z񐔂̐؂Ȃ
	if(Npos_ini != Npos_cur){
		Initialize_Change(Nmic,Npos_cur,iLower_f,iUpper_f) ;
		iRtn = 1 ;
	}

	//ETF_CONJ̍XV
	Cexec ::AssignSrcPos(plaSrcPosCur) ;	

	//TF_CONJ̍XV
	Cexec :: InitTFMatrix(&psPrmFrame) ;
	bRtn = false ; 
	if(m_iUpdateMethodTF_CONJ == __UPDATE_METHOD_ID){
		bRtn = _CheckSrcID_TF_CONJ(tmp_cmpaTFDB,plaSrcPosIni) ;
	}
	
	//WXVpɈxɖ߂
	Cexec :: AssignSrcPos(plaSrcPosIni) ;
	
	ComplexArrayFree(&tmp_cmpaTFDB) ;
	LocationArrayFree(&plaSrcPosIni) ;

	if(iRtn == 1){	//ύX
		return 1 ;
	}
	else{
		if(bRtn == false){ //ʒuύXA͕s
			return 2 ;
		}
		else{	//ʒuCύXȂ
			return 0 ;
		}

	}
	
}
// ---------------------------------------------------------------------------------------
bool Cexec::_CheckSrcID_TF_CONJ(MICARY_ComplexArray caTFDBInit,MICARY_LocationArray laSrcPosInit)	
{
	int i,j,k,m,Npos_cur,Npos_init,Nmic,Nfreq,iLower_f,iUpper_f ;
	int  iInherit ;
	
	Npos_cur = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ;
	Npos_init = laSrcPosInit.ha_numLocation ;
	Nmic = m_psSepMat->micary_smSeparation.ha_numInput ;
	iLower_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_iValue[0] ;
	iUpper_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue + iLower_f -1 ;
	Nfreq = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;
	
	iInherit = 0 ;
	//TF_CONJf[^̈p/
	for(i = 0 ; i < Npos_cur ; i++){
	  for(j = 0 ; j < Npos_init ; j++){
	    // ̈ʒu(p)
	    if(m_psSepMat->micary_tdTFDB.micary_locaSource.ha_idLocation[i] == laSrcPosInit.ha_idLocation[j]){
			for(k = 0 ; k < Nfreq ; k++){
				for(m = 0 ; m < Nmic ; m++){
					m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_cmpValue[m+i*Nmic+k*Nmic*Npos_cur].re 
						= caTFDBInit.ha_cmpValue[m+j*Nmic+k*Nmic*Npos_init].re ;
		   			m_psSepMat->micary_tdTFDB.micary_cmpaTF.ha_cmpValue[m+i*Nmic+k*Nmic*Npos_cur].im 
						= caTFDBInit.ha_cmpValue[m+j*Nmic+k*Nmic*Npos_init].im ;
				}
			}
			iInherit++ ;
			
			break ;
		}
	  } 

	}
	if(iInherit == Npos_cur){	//ύXȂ
		return true ;
	}
	else{	//ύX
		return false ;
	}
	
}
//-------------------------------------------------------------------------------------------
int Cexec::ChangeSrcPos_W(MICARY_LocationArray plaSrcPosCur, MICARY_PrmFrame psPrmFrame)
{
	int i ;
	int Npos_ini,Npos_cur,Nmic,Nfreq,iLower_f,iUpper_f ;
	bool bRtn ,bRtnID;
	int iRtn = 2 ;

	Nmic = m_psSepMat->micary_smSeparation.ha_numInput ;
	Npos_ini = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ;
	iLower_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_iValue[0] ;
	iUpper_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue + iLower_f -1 ;
	Nfreq = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;
	Npos_cur = plaSrcPosCur.ha_numLocation ;
	
			
	//z񐔂̐؂Ȃ
	if(Npos_ini != Npos_cur){
		iRtn = 1 ;
	}
	if(m_iUpdateMethodWex->ha_numValue != Npos_cur){
		IntArrayFree(m_iUpdateMethodWex) ;
		IntArrayAlloc(m_iUpdateMethodWex,Npos_cur) ;
	}
	
	// Change_Src_Pos_TF_CONJŐ؂蒼Ă͂
	if(m_psSepMat->micary_smSeparation.micary_cmpaSeparation.ha_numValue != Npos_cur * Nfreq*Nmic){
		SeparationMatFree(&m_psSepMat->micary_smSeparation) ;
		SeparationMatAlloc(&m_psSepMat->micary_smSeparation,Nfreq,Npos_cur,Nmic) ;
	}

	//̍XV
	Cexec ::AssignSrcPos(plaSrcPosCur) ;
	// s̍XV
	Cexec ::InitSepMatrix(&psPrmFrame) ;
	
	bRtn = false ; 
	bRtnID = false ;
	int iInherit ;
	iInherit = 0 ;
	//s̈pmF
	for(i = 0 ; i < Npos_cur ; i++){
		m_iUpdateMethodWex->ha_iValue[i] = m_iUpdateMethodW ;
		
		if(m_iUpdateMethodW == __UPDATE_METHOD_ID 
				|| m_iUpdateMethodW == __UPDATE_METHOD_ID_POS){
			// IDx[X̊mF
			bRtnID = _CheckSrcID_W(i) ;
			if(bRtnID == true){
				iInherit++ ;
			}
		}
		//ۂ̍XVID or POS
		if(m_iUpdateMethodW == __UPDATE_METHOD_ID_POS){
			if(bRtnID == false){
				m_iUpdateMethodWex->ha_iValue[i] = __UPDATE_METHOD_POS ;
			}
			else{
				m_iUpdateMethodWex->ha_iValue[i] = __UPDATE_METHOD_ID ;
			}
		}

		if(m_iUpdateMethodWex->ha_iValue[i] == __UPDATE_METHOD_POS){
			//POSx[X̊mF
			bRtn = _CheckSrcPos_W(i) ;
			if(bRtn == true){
				iInherit++ ;
			}

			else{
			  if(m_iUpdateMethodW == __UPDATE_METHOD_ID_POS){
			    m_iUpdateMethodWex->ha_iValue[i] = __UPDATE_METHOD_ID ;
			  }
			}

			
		}
		
	}
	if(iRtn == 1){	//AύX
		return 1 ;
	}
	else{
		if(iInherit == Npos_cur){	//SĕύXȂ
			return 0 ;
		}
		else{	//ꕔύX
			return 2 ;
		}
	}
		
}

// ----------------------------------------------------------------
bool Cexec::_CheckSrcID_W(int iIdcur)
{
	int k,m,Npos_cur,Npos_init,Nmic,Nfreq,iLower_f,iUpper_f ;
	int iInherit ;
	int iIdsave ;

	Npos_cur = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ;
	Npos_init = m_updateW->micary_dbSeparation.micary_locaSource.ha_numLocation ;
	Nmic = m_psSepMat->micary_smSeparation.ha_numInput ;
	iLower_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_iValue[0] ;
	iUpper_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue + iLower_f -1 ;
	Nfreq = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;
	
	//Ot[̉IDƔr(Create or Exist)
	iIdsave = _CheckFrontSrcLoc(m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[iIdcur],__UPDATE_METHOD_ID) ;
	if(iIdsave == __INVALID_SRC_ID){
		return false ;
	}
	iInherit = 0 ;
	if(m_updateW->micary_iaUpdate.ha_iValue[iIdsave] == __SRCID_FRONT_EXIST){	
	//s̈p
		for(k = 0 ; k < Nfreq ; k++){
			for(m = 0 ; m < Nmic ; m++){
				m_psSepMat->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iIdcur+m*Npos_cur+k*Nmic*Npos_cur].re 
					= m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iIdsave+m*Npos_init+k*Nmic*Npos_init].re ;
		   		m_psSepMat->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iIdcur+m*Npos_cur+k*Nmic*Npos_cur].im 
					= m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iIdsave+m*Npos_init+k*Nmic*Npos_init].im ;
		   	
			}
		}
		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[iIdcur].ha_fX 
			= m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iIdsave].ha_fX ;
		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[iIdcur].ha_fY 
			= m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iIdsave].ha_fY ;
		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[iIdcur].ha_fZ 
			= m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iIdsave].ha_fZ ;
		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[iIdcur].ha_idPos
			= m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iIdsave].ha_idPos ;
		m_psSepMat->micary_tdTFDB.micary_locaSource.ha_idLocation[iIdcur] 
			= m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[iIdsave] ;	
			
		iInherit++ ;
	}
	
	if(iInherit == 1){	//ύXȂ(Ot[p)
		return true ;
	}
	else{
		return false ;
	}
}

// -----------------------------------------------------------------
bool Cexec::_CheckSrcPos_W(int iIdcur)
{
	int k,m,Npos_cur,Npos_init,Nmic,Nfreq,iLower_f,iUpper_f ;
	bool btmp;
	int iIDpos,iInherit ;
	//MICARY_Location locPos ;

//printf("<CheckSrcPos:");
	Npos_cur = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ;
	Npos_init = m_updateW->micary_dbSeparation.micary_locaSource.ha_numLocation ;
	Nmic = m_psSepMat->micary_smSeparation.ha_numInput ;
	iLower_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_iValue[0] ;
	iUpper_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue + iLower_f -1 ;
	Nfreq = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;

	//Ot[̉IDƔr(Create or Exist)
	iIDpos = _CheckFrontSrcLoc(m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[iIdcur],__UPDATE_METHOD_POS) ;
	if(iIDpos == __INVALID_SRC_ID){
//printf("10>");
		return false ;
	}
	
//printf("20,");
	iInherit = 0 ;

	if(m_updateW->micary_iaUpdate.ha_iValue[iIDpos] == __SRCID_FRONT_EXIST){
		for(k = 0 ; k < Nfreq ; k++){
			for(m = 0 ; m < Nmic ; m++){
				m_psSepMat->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iIdcur+m*Npos_cur+k*Nmic*Npos_cur].re 
						= m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iIDpos+m*Npos_init+k*Nmic*Npos_init].re ;
		   		m_psSepMat->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iIdcur+m*Npos_cur+k*Nmic*Npos_cur].im 
						= m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iIDpos+m*Npos_init+k*Nmic*Npos_init].im ;
		   	
			}
		}
		
		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[iIdcur].ha_fX 
			= m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iIDpos].ha_fX ;
		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[iIdcur].ha_fY 
			= m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iIDpos].ha_fY ;
		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[iIdcur].ha_fZ 
			= m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iIDpos].ha_fZ ;
		m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[iIdcur].ha_idPos
			= m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iIDpos].ha_idPos ;
		m_psSepMat->micary_tdTFDB.micary_locaSource.ha_idLocation[iIdcur] 
			= m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[iIDpos] ;	

		iInherit++ ;
	
	}
	
	if(iInherit == 1){	//ύXȂ
//printf("30(iIDpos:%d)>",iIDpos);
		return true ;
	}
	else{
//printf("40>");
		return false ;
	}

}
/*
void Cexec::SetUpdateW(int iIdcur,int iSrc, MICARY_Location locPos, int iUpdateMethod)
{
	int k,m,iPos,Nmic,Npos_cur,Npos_all,iLower_f,iUpper_f,Nfreq;


	Nmic = m_psSepMat->micary_smSeparation.ha_numInput ;
	Npos_cur = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ;
	Npos_all = m_updateW->micary_dbSeparation.micary_locaSource.ha_numLocation ;
	iLower_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_iValue[0] ;
	iUpper_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue + iLower_f -1 ;
	Nfreq = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;
	
	switch (iUpdateMethod){
	case __UPDATE_METHOD_POS :
			iPos = GetNearestPos(locPos,&(m_updateW->micary_dbSeparation.micary_locaSource)) ;
			if(iPos == __INVALID_SRC_ID){
				return ;
			}
	break ;
	case __UPDATE_METHOD_ID :
		for(iPos = 0 ; iPos < Npos_all ; iPos++){
			if(m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[iPos] == iSrc){
				break ;
			}
		}
	break ;
	}
	for(k = 0 ; k < Nfreq ; k++){
		for(m = 0 ; m < Nmic ; m++){
			m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iPos+m*Npos_all+k*Npos_all*Nmic].re=
					m_psSepMat->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iIdcur+m*Npos_cur+k*Npos_cur*Nmic].re ;
			m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iPos+m*Npos_all+k*Npos_all*Nmic].im = 
					m_psSepMat->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iIdcur+m*Npos_cur+k*Npos_cur*Nmic].im  ;
		}
	}
	//̍XV
	m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iPos].ha_fX = locPos.ha_fX ;
	m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iPos].ha_fY = locPos.ha_fY ;
	m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iPos].ha_fZ = locPos.ha_fZ ;

	//IDXV
	m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[iPos]= iSrc ;
	m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iPos].ha_idPos = iSrc ;

	//XV̍XV
	m_updateW->micary_iaUpdate.ha_iValue[iPos] = __SRCID_FRONT_EXIST ; //2009/8/5
	
	return ;

}
*/
// ----------------------------------------------------------------------------------------
bool Cexec::_IsThreshPos(MICARY_Location laRefPos,MICARY_Location laObjPos)
{
	// e͈͂̓ςvZ
	MICARY_Location laThresOrg,laThresAng ;
	float fPOrg ,fPObj;

	// _0
	laThresOrg.ha_fPhi = 16.0 ;
	laThresOrg.ha_fTheta = 0.0 ;
	_Ang2Pos(&laThresOrg) ;

	// epx
	laThresAng.ha_fPhi = 16.0 ;
	laThresAng.ha_fTheta = m_fThred_angle ; 
	_Ang2Pos(&laThresAng) ;

	// epx̓
	fPOrg = _InnerProductLoc(laThresOrg,laThresAng) ;

	// ړIpx̓ςvZ
	fPObj = _InnerProductLoc(laRefPos,laObjPos) ;

	//ς̔r
	if(fPObj >= fPOrg){
		return true ;
	}
	else{
		return false ;
	}
}
// ------------------------------------------------------------------------------------
void Cexec::_Ang2Pos(MICARY_Location *plocPos)
{
	float ftheta,fphi ;
	float pi ;

	pi = acos(-1.0) ;
	ftheta = plocPos->ha_fTheta*pi/180.0 ;
	fphi = plocPos->ha_fPhi*pi/180.0 ;

	plocPos->ha_fX = cos(ftheta)*cos(fphi) ;
	plocPos->ha_fY = sin(ftheta)*cos(fphi) ;
	plocPos->ha_fZ = sin(fphi) ;

	return ;

}
// -----------------------------------------------------------------------------------
HA_Float Cexec::_InnerProductLoc(MICARY_Location micary_Loc1, MICARY_Location micary_Loc2)
{
	HA_Float ha_fNorm,ha_fP ;
	MICARY_FloatArray laVec1 , laVec2 ;
	
	FloatArrayAlloc(&laVec1,0) ;
	FloatArrayAlloc(&laVec2,0) ;

	FloatArrayAlloc(&laVec1,3) ;
	FloatArrayAlloc(&laVec2,3) ;

	// K
	ha_fNorm = pow((double)micary_Loc1.ha_fX,2.0) + pow((double)micary_Loc1.ha_fY,2.0)+pow((double)micary_Loc1.ha_fZ,2.0) ;
	ha_fNorm = pow((double)ha_fNorm,0.5) ;
	
	laVec1.ha_fValue[0] = micary_Loc1.ha_fX / ha_fNorm ;
	laVec1.ha_fValue[1] = micary_Loc1.ha_fY / ha_fNorm ;
	laVec1.ha_fValue[2] = micary_Loc1.ha_fZ / ha_fNorm ;


	// K
	ha_fNorm = pow((double)micary_Loc2.ha_fX,2.0) + pow((double)micary_Loc2.ha_fY,2.0)+pow((double)micary_Loc2.ha_fZ,2.0) ;
	ha_fNorm = pow((double)ha_fNorm,0.5) ;
	
	laVec2.ha_fValue[0] = micary_Loc2.ha_fX / ha_fNorm ;
	laVec2.ha_fValue[1] = micary_Loc2.ha_fY / ha_fNorm ;
	laVec2.ha_fValue[2] = micary_Loc2.ha_fZ / ha_fNorm ;

	ha_fP = _InnerProductVec(laVec1,laVec2) ;
	
	FloatArrayFree(&laVec1) ;
	FloatArrayFree(&laVec2) ;
	
	return ha_fP ;
	
}
// -------------------------------------------------------
bool Cexec::ExportUpdateW(MICARY_PrmFrame *psPrmFrame,const char *cFilename)
{
  int i,iNvalidSize,iNmic,iNfreq,iNpos,iiv,k,m ;
	
		
	iNvalidSize = _GetNvalidUpdateW() ;
	if(iNvalidSize == 0){
		return false ;
	}
	iNmic = m_updateW->micary_dbSeparation.micary_smSeparation.ha_numInput ;
	iNpos = m_updateW->micary_dbSeparation.micary_smSeparation.ha_numOutput ;
	iNfreq = m_updateW->micary_dbSeparation.micary_smSeparation.micary_iaFreqLinex.ha_numValue ;
	MICARY_SMDatabase *psValidW ;
	psValidW = (MICARY_SMDatabase *)calloc(1,sizeof(MICARY_SMDatabase)) ;
	SMDatabaseAlloc(psValidW,0,0,0) ;
	SMDatabaseAlloc(psValidW,iNfreq ,iNvalidSize,iNmic) ;

	//}CNʒȗ
	for(i = 0 ; i < iNmic ; i++){
		psValidW->micary_locaMic.micary_Location[i].ha_fX = m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[i].ha_fX ;
		psValidW->micary_locaMic.micary_Location[i].ha_fY =m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[i].ha_fY ;
		psValidW->micary_locaMic.micary_Location[i].ha_fZ = m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[i].ha_fZ;
		psValidW->micary_locaMic.micary_Location[i].ha_idPos = m_updateW->micary_dbSeparation.micary_locaMic.micary_Location[i].ha_idPos;
		psValidW->micary_locaMic.ha_idLocation[i] = m_updateW->micary_dbSeparation.micary_locaMic.ha_idLocation[i];

	}


	// gC̑
	for(i = 0 ; i < iNfreq; i++){
		psValidW->micary_smSeparation.micary_iaFreqLinex.ha_iValue[i] = m_updateW->micary_dbSeparation.micary_smSeparation.micary_iaFreqLinex.ha_iValue[i];
	}

	iiv = 0 ;
	
	for(i = 0; i < iNpos ; i++){
		if(m_updateW->micary_iaUpdate.ha_iValue[i] == __SRCID_FRONT_CREATE || m_updateW->micary_iaUpdate.ha_iValue[i] == __SRCID_FRONT_EXIST){
			psValidW->micary_locaSource.ha_idLocation[iiv] = m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[i];
			psValidW->micary_locaSource.micary_Location[iiv].ha_fX = m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[i].ha_fX ;
			psValidW->micary_locaSource.micary_Location[iiv].ha_fY =m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[i].ha_fY ;
			psValidW->micary_locaSource.micary_Location[iiv].ha_fZ = m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[i].ha_fZ;
			psValidW->micary_locaSource.micary_Location[iiv].ha_idPos = m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[i].ha_idPos ;
		
			
			for(k = 0 ; k < iNfreq ; k++){
				for(m = 0 ; m < iNmic ; m++){
					psValidW->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iiv+m*iNvalidSize+k*iNmic*iNvalidSize].re 
							= m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[i+m*iNpos+k*iNmic*iNpos].re ;
		   			psValidW->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iiv+m*iNvalidSize+k*iNmic*iNvalidSize].im 
							= m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[i+m*iNpos+k*iNmic*iNpos].im ;
		   	
				}
			}
			iiv++ ;
			if(iiv == iNvalidSize){
			  break ;
			}
		}
	}
	
	HA_String file ; // TF t@C
	file.s = (HA_Char *)calloc(strlen(cFilename)+1,sizeof(char)) ;
	strcpy(file.s,cFilename) ;
	file.slen = strlen(cFilename) ;
	if(m_iFormat_ExportW == __FILE_FORMAT_HARK){
		MICARY_TFDBFileHeader *psH ;
		psH = new MICARY_TFDBFileHeader() ;
		psH->lFs = (int)psPrmFrame->ha_fFs ;
		psH->lNFFT = psPrmFrame->ha_numNFFT ;
		psH->lNMic = iNmic ;
		psH->lNSrc = iNpos ;
		psH->lNFreq = iNfreq ;
		psH->iVersion = 1 ;
		psH->iFileType = FORMAT_TYPE_W ;

		if(!cHark.WriteData(file.s,psH,
					psValidW->micary_smSeparation.micary_cmpaSeparation,
					psValidW->micary_smSeparation.micary_iaFreqLinex,
					psValidW->micary_locaSource,
					psValidW->micary_locaMic,
					"",
					"",
					"",
					"",
					"",
				    "GHDSS_EXPORT")){
			if(file.s){
				free(file.s) ;
				file.s = NULL ;
			}
			SMDatabaseFree(psValidW) ;
			free(psValidW);
			psValidW = NULL ;
			if(psH){
				delete [] psH ;
				psH = NULL ;
			}
			return false ;
		}

	}


	if(file.s){
		free(file.s) ;
		file.s = NULL ;
	}
	SMDatabaseFree(psValidW) ;
	free(psValidW) ;
	psValidW = NULL ;
	return true ;

}
// -------------------------------------------------------
int Cexec::_GetNvalidUpdateW()
{
	int iRtn,i ;
	iRtn = 0 ;
	//for(i = 0 ; i < m_updateW->micary_iaUpdate.ha_numValue ; i++){
	for(i = 0 ; i < m_updateW->micary_dbSeparation.micary_locaSource.ha_numLocation ; i++){
		if(m_updateW->micary_iaUpdate.ha_iValue[i] != __INVALID_SRC_ID){
			iRtn++ ;
		}
	}
	return iRtn ;

}
// ---------------------------------------------------------
void Cexec::SetUpdateW()
{
	int ii,jj,k,m,iPos,Nmic,Npos_cur,Npos_all,iLower_f,iUpper_f,Nfreq;

	Nmic = m_psSepMat->micary_smSeparation.ha_numInput ;
	Npos_cur = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ;
	Npos_all = m_updateW->micary_dbSeparation.micary_locaSource.ha_numLocation ;
	iLower_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_iValue[0] ;
	iUpper_f = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue + iLower_f -1 ;
	Nfreq = m_psSepMat->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;

	if(m_iUpdateMethodWex->ha_numValue == 0){
		// 2011/11/16 changed by N.Tanaka
		//IntArrayFree(m_iUpdateMethodWex) ;
		//IntArrayAlloc(m_iUpdateMethodWex,Npos_cur) ;
		IntArrayClear(m_iUpdateMethodWex) ;
		IntArrayAlloc_r(m_iUpdateMethodWex, Npos_cur, __MICARY_def_MAX_NSRC) ;
		for(ii = 0 ; ii < Npos_cur ; ii++){
			m_iUpdateMethodWex->ha_iValue[ii] = m_iUpdateMethodW ;
		}
	}

	for(ii = 0 ; ii < Npos_cur ; ii++){
		switch (m_iUpdateMethodWex->ha_iValue[ii]){
		case __UPDATE_METHOD_POS :
			iPos = GetNearestPos(m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[ii],
				&(m_updateW->micary_dbSeparation.micary_locaSource)) ;
			if(iPos == __INVALID_SRC_ID){
			  for(jj = 0 ; jj < Npos_all ; jj++){
			    if(m_updateW->micary_iaUpdate.ha_iValue[jj] == __INVALID_SRC_ID){
				iPos = jj ;
				break ;
			    }
			  }
			}	  
			for(k = 0 ; k < Nfreq ; k++){
					for(m = 0 ; m < Nmic ; m++){
						m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iPos+m*Npos_all+k*Npos_all*Nmic].re=
							m_psSepMat->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[ii+m*Npos_cur+k*Npos_cur*Nmic].re ;
						m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iPos+m*Npos_all+k*Npos_all*Nmic].im = 
							m_psSepMat->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[ii+m*Npos_cur+k*Npos_cur*Nmic].im  ;
					}
				}
				//̍XV
				m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iPos].ha_fX = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[ii].ha_fX ;
				m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iPos].ha_fY = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[ii].ha_fY ;
				m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iPos].ha_fZ = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[ii].ha_fZ ;
				m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iPos].ha_idPos = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[ii].ha_idPos ;

				//IDXV
				m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[iPos]= m_psSepMat->micary_tdTFDB.micary_locaSource.ha_idLocation[ii] ;
				//XV̍XV
				m_updateW->micary_iaUpdate.ha_iValue[iPos] = __SRCID_FRONT_EXIST ;

		break ;
		case __UPDATE_METHOD_ID :
		case __UPDATE_METHOD_ID_POS :
		  m_iUpdateMethodWex->ha_iValue[ii] = __UPDATE_METHOD_ID;
		  iPos = Npos_all ;
			for(jj = 0 ; jj < Npos_all ; jj++){
				if(m_psSepMat->micary_tdTFDB.micary_locaSource.ha_idLocation[ii] ==
					m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[jj]){
						iPos = jj ;
						break ;
				}
			}
			if(iPos >= Npos_all){
				for(jj = 0 ; jj < Npos_all ; jj++){
					if(m_updateW->micary_iaUpdate.ha_iValue[jj] == __INVALID_SRC_ID){
						iPos = jj ;
						break ;
					}
				}
			}
		

			for(k = 0 ; k < Nfreq ; k++){
				for(m = 0 ; m < Nmic ; m++){
					m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iPos+m*Npos_all+k*Npos_all*Nmic].re=
						m_psSepMat->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[ii+m*Npos_cur+k*Npos_cur*Nmic].re ;
					m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[iPos+m*Npos_all+k*Npos_all*Nmic].im = 
						m_psSepMat->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[ii+m*Npos_cur+k*Npos_cur*Nmic].im  ;
				}
			}
					
			//̍XV
			m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iPos].ha_fX = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[ii].ha_fX ;
			m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iPos].ha_fY = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[ii].ha_fY ;
			m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iPos].ha_fZ = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[ii].ha_fZ ;

			//IDXV
			m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[iPos] = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_idLocation[ii] ;
			m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[iPos].ha_idPos = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[ii].ha_idPos ;

			//XV̍XV
			m_updateW->micary_iaUpdate.ha_iValue[iPos] = __SRCID_FRONT_EXIST ;

		break ;
		default :
		break ;
		}
	}
	_SetRemoveSrc() ;

	return ;

}
// -----------------------------------------------------------------------------------------------
void Cexec::_SetRemoveSrc()
{
	//ł̊mF
	int ii,jj,kk ;
	bool bRemove = true ;

	for(ii = 0 ; ii < m_laFrontSrcLoc->ha_numLocation ; ii++){

  		switch(m_laFrontSrcLoc->micary_Location[ii].micary_LocationFlag){
		case __UPDATE_METHOD_POS :
	
			jj = GetNearestPos(m_laFrontSrcLoc->micary_Location[ii],&m_psSepMat->micary_tdTFDB.micary_locaSource) ;
			if(jj != __INVALID_SRC_ID){
				bRemove = _IsThreshPos(m_laFrontSrcLoc->micary_Location[ii],m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[jj]) ;
			
				if(!bRemove){
					kk = GetNearestPos(m_laFrontSrcLoc->micary_Location[ii],&m_updateW->micary_dbSeparation.micary_locaSource) ;
					if(kk!=__INVALID_SRC_ID){
						m_updateW->micary_iaUpdate.ha_iValue[kk] = __SRCID_FRONT_REMOVE;
						#ifdef __DEBUG_UPDATEW
						cout << "Src REMOVE " << endl ;
						#endif
					
					}
				}
			}
	       

		break ;
		case __UPDATE_METHOD_ID :

	
			bRemove = true ;
			for(jj = 0 ; jj < m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ; jj++){
				if(m_laFrontSrcLoc->ha_idLocation[ii] == m_psSepMat->micary_tdTFDB.micary_locaSource.ha_idLocation[jj]){
					bRemove = false ;
					break ;
				}
			}
			if(bRemove){	//Ot[IDIDɂȂ=łꍇ
				for(kk = 0 ; kk < m_updateW->micary_dbSeparation.micary_locaSource.ha_numLocation ; kk++){
					if(m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[kk] == m_laFrontSrcLoc->ha_idLocation[ii]){
						m_updateW->micary_iaUpdate.ha_iValue[kk] = __SRCID_FRONT_REMOVE;
					
						break ;
					}
				}
			}
		break ;
		case __UPDATE_METHOD_ID_POS :

			bRemove = true ;
			for(jj = 0 ; jj < m_psSepMat->micary_tdTFDB.micary_locaSource.ha_numLocation ; jj++){
				if(m_laFrontSrcLoc->ha_idLocation[ii] == m_psSepMat->micary_tdTFDB.micary_locaSource.ha_idLocation[jj]){
					bRemove = false ;
					break ;
				}
			}
			if(bRemove){
				jj = GetNearestPos(m_laFrontSrcLoc->micary_Location[ii],&m_psSepMat->micary_tdTFDB.micary_locaSource) ;
				if(jj != __INVALID_SRC_ID){
					bool btmp=_IsThreshPos(m_laFrontSrcLoc->micary_Location[ii],m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[jj]) ;
					if(btmp){
						bRemove = false ;
					}
				}
			}
			if(bRemove){	//
				kk = GetNearestPos(m_laFrontSrcLoc->micary_Location[ii],&m_updateW->micary_dbSeparation.micary_locaSource) ;
				if(kk!=__INVALID_SRC_ID){
					m_updateW->micary_iaUpdate.ha_iValue[kk] = __SRCID_FRONT_REMOVE;
				}
			}
	     
		
		break ;
		}
	}
	
	// zTCY̐؂Ȃ
	if(m_laFrontSrcLoc->ha_numLocation != m_psSepMat->micary_smSeparation.ha_numOutput){
		// 2011/11/16 changed by N.Tanaka
		//LocationArrayFree(m_laFrontSrcLoc) ;
		//LocationArrayAlloc(m_laFrontSrcLoc,m_psSepMat->micary_smSeparation.ha_numOutput) ;
		LocationArrayClear(m_laFrontSrcLoc) ;
		LocationArrayAlloc_r(m_laFrontSrcLoc,m_psSepMat->micary_smSeparation.ha_numOutput, __MICARY_def_MAX_NSRC) ;
	}
	for(ii = 0 ; ii < m_laFrontSrcLoc->ha_numLocation ; ii++){
		m_laFrontSrcLoc->ha_idLocation[ii] = m_psSepMat->micary_tdTFDB.micary_locaSource.ha_idLocation[ii] ;
		m_laFrontSrcLoc->micary_Location[ii].ha_fX = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[ii].ha_fX ;
		m_laFrontSrcLoc->micary_Location[ii].ha_fY = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[ii].ha_fY ;
		m_laFrontSrcLoc->micary_Location[ii].ha_fZ = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[ii].ha_fZ ;
		m_laFrontSrcLoc->micary_Location[ii].ha_idPos = m_psSepMat->micary_tdTFDB.micary_locaSource.micary_Location[ii].ha_idPos;

		// Update@XV
		m_laFrontSrcLoc->micary_Location[ii].micary_LocationFlag = m_iUpdateMethodWex->ha_iValue[ii] ;

	}
	
}

// -----------------------------------------------------------------------------------------------
// Ot[̉Ɣr
int Cexec::_CheckFrontSrcLoc(MICARY_Location lcCurLoc,int iUpdateMethodW)
{
	int ii,jj ;
	bool btmp ;
	switch (iUpdateMethodW){
	case __UPDATE_METHOD_POS :
		//t[ƂPt[OPosition
		ii = GetNearestPos(lcCurLoc,m_laFrontSrcLoc) ;
//printf("<_CheckFrontSrcLoc:10(%d),", ii);
		if(ii != __INVALID_SRC_ID){	//ꍇ
			//e͈͓mF
			btmp = _IsThreshPos(lcCurLoc,m_laFrontSrcLoc->micary_Location[ii]) ;
			if(btmp == true){	//e͈͓(݂Ă)Ȃ΁Cۑlf[^Ō
				// 1t[ỎʒuŌ
				jj = GetNearestPos(m_laFrontSrcLoc->micary_Location[ii],&m_updateW->micary_dbSeparation.micary_locaSource) ;
				if(jj != __INVALID_SRC_ID){
					m_updateW->micary_iaUpdate.ha_iValue[jj] = __SRCID_FRONT_EXIST ;
					//ۑlްt[ʒuɍXV
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_fX = lcCurLoc.ha_fX ;
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_fY = lcCurLoc.ha_fY ;
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_fZ = lcCurLoc.ha_fZ ;
					m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[jj] = m_laFrontSrcLoc->micary_Location[ii].ha_idPos ;
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_idPos = m_laFrontSrcLoc->micary_Location[ii].ha_idPos ;
//printf("20(%d)>",jj);
					
					return jj ;
				}
			}
			//1t[ỎƂ͈قȂꍇCȑOɏoʒuHiۑlf[^Ōj
			for(jj = 0 ; jj < m_updateW->micary_dbSeparation.micary_locaSource.ha_numLocation ; jj++){
				if(m_updateW->micary_iaUpdate.ha_iValue[jj] != __INVALID_SRC_ID){
			
					btmp = _IsThreshPos(lcCurLoc,m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj]) ;
						if(btmp == true){	// ۑlf[^ŌꍇCXV
							m_updateW->micary_iaUpdate.ha_iValue[jj] = __SRCID_FRONT_EXIST ; //
							m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_fX = lcCurLoc.ha_fX ;
							m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_fY = lcCurLoc.ha_fY ;
							m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_fZ = lcCurLoc.ha_fZ ;
							
//printf("30(%d)>",jj);
							return jj ;
						}
				}
			}
			
			// 1t[Oɂ͂Ȃ=CREATEȂ΁AVKɕۑꏊ̔ԍԂ
			for(jj = 0 ; jj < m_updateW->micary_dbSeparation.micary_locaSource.ha_numLocation ; jj++){
				if(m_updateW->micary_iaUpdate.ha_iValue[jj] == __INVALID_SRC_ID){
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_fX = lcCurLoc.ha_fX ;
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_fY = lcCurLoc.ha_fY ;
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_fZ = lcCurLoc.ha_fZ ;
					m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[jj] = lcCurLoc.ha_idPos ;
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_idPos = lcCurLoc.ha_idPos ;
					
					m_updateW->micary_iaUpdate.ha_iValue[jj] = __SRCID_FRONT_CREATE ;
				
//printf("40(%d)>",jj);
					return jj ;
				}
			}
		}
//printf("50>");
		return __INVALID_SRC_ID ;

	break ;
	case __UPDATE_METHOD_ID :
		for(ii = 0 ; ii < m_laFrontSrcLoc->ha_numLocation ; ii++){
			if(m_laFrontSrcLoc->ha_idLocation[ii] == lcCurLoc.ha_idPos){ //1t[ỎƈvĂ
			  for(jj = 0 ; jj < m_updateW->micary_dbSeparation.micary_locaSource.ha_numLocation ; jj++){//ۑlf[^̔ԍԂ
			    	if(m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[jj] == lcCurLoc.ha_idPos){
			    	  if(m_updateW->micary_iaUpdate.ha_iValue[jj] == __SRCID_FRONT_CREATE || 
							m_updateW->micary_iaUpdate.ha_iValue[jj] == __SRCID_FRONT_EXIST){
										
							m_updateW->micary_iaUpdate.ha_iValue[jj] = __SRCID_FRONT_EXIST ;
							return jj ;
						}
					}
				}
			}
		}
		if(m_updateW == __UPDATE_METHOD_ID){	//add 2009.7.29
			// 1t[Ỏɂ͂Ȃ=CREATEȂ΁AVKɕۑꏊ̔ԍԂ
			for(jj = 0 ; jj < m_updateW->micary_dbSeparation.micary_locaSource.ha_numLocation ; jj++){
				if(m_updateW->micary_iaUpdate.ha_iValue[jj] == __INVALID_SRC_ID){
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_fX = lcCurLoc.ha_fX ;
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_fY = lcCurLoc.ha_fY ;
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_fZ = lcCurLoc.ha_fZ ;
					
					m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[jj] = lcCurLoc.ha_idPos ;
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_idPos = lcCurLoc.ha_idPos ;
					m_updateW->micary_iaUpdate.ha_iValue[jj] = __SRCID_FRONT_CREATE ;
				
					
					return jj ;
				}
			}
		}
		return	__INVALID_SRC_ID ; // error code
	break ;

	case __UPDATE_METHOD_ID_POS :
		for(ii = 0 ; ii < m_laFrontSrcLoc->ha_numLocation ; ii++){
			if(m_laFrontSrcLoc->ha_idLocation[ii] == lcCurLoc.ha_idPos){ //1t[ỎƈvĂ
				for(jj = 0 ; jj < m_updateW->micary_dbSeparation.micary_locaSource.ha_numLocation ; jj++){//ۑlf[^̔ԍԂ
					if(m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[jj] == lcCurLoc.ha_idPos){
						m_updateW->micary_iaUpdate.ha_iValue[jj] = __SRCID_FRONT_EXIST ;
						return jj ;
					}
				}
			}
		}
		// 1t[Ỏɂ͂Ȃ=CREATEȂ΁APOSŌ
		ii = GetNearestPos(lcCurLoc,m_laFrontSrcLoc) ;
		//e͈͓̊mF
		if(ii != __INVALID_SRC_ID){
			btmp = _IsThreshPos(lcCurLoc,m_laFrontSrcLoc->micary_Location[ii]) ;
			if(btmp == true){	//e͈͓(݂Ă)Ȃ΁Cۑlf[^Ō
				// 1t[ỎʒuŌ
				jj = GetNearestPos(m_laFrontSrcLoc->micary_Location[ii],&m_updateW->micary_dbSeparation.micary_locaSource) ;
				if(jj != __INVALID_SRC_ID){
					m_updateW->micary_iaUpdate.ha_iValue[jj] = __SRCID_FRONT_EXIST ;
				}
				return jj ;
			}
			// 1t[Ỏɂ͂Ȃ=CREATEȂ΁AVKɕۑꏊ̔ԍԂ
			for(jj = 0 ; jj < m_updateW->micary_dbSeparation.micary_locaSource.ha_numLocation ; jj++){
				if(m_updateW->micary_iaUpdate.ha_iValue[jj] == __INVALID_SRC_ID){
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_fX = lcCurLoc.ha_fX ;
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_fY = lcCurLoc.ha_fY ;
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_fZ = lcCurLoc.ha_fZ ;
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[jj].ha_idPos = lcCurLoc.ha_idPos ;
					m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[jj] = lcCurLoc.ha_idPos ;
					m_updateW->micary_iaUpdate.ha_iValue[jj] = __SRCID_FRONT_CREATE ;
					
					return jj ;
				}
			}
		}
		return __INVALID_SRC_ID ;

	break ;
	}

}
// ---------------------------------------------------------------------------
bool Cexec::AddUpdateW(MICARY_PrmFrame *psPrmFrame,const char *cFilename,bool bExpW)
{
	int ii ,jj ,Npos,Nmic,Nfreq,k,m,Npos_all,idtmp;
	bool bUpdate ;
	bool bRtn ;

	Npos_all = m_updateW->micary_dbSeparation.micary_locaSource.ha_numLocation;
	Nmic = m_updateW->micary_dbSeparation.micary_locaMic.ha_numLocation ;
	Nfreq = m_updateW->micary_dbSeparation.micary_smSeparation.micary_iaFreqLinex.ha_numValue ;

	for(ii = 0 ; ii < Npos_all ;ii++){
		if(m_updateW->micary_iaUpdate.ha_iValue[ii] == __SRCID_FRONT_REMOVE && 
			m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_idPos != __INVALID_SRC_ID){
		      if(!bExpW){
		    	//tO̍XV
		    	if(m_iUpdateMethodW == __UPDATE_METHOD_ID || m_iUpdateMethodW == __UPDATE_METHOD_ID_POS){
			       m_updateW->micary_iaUpdate.ha_iValue[ii]=__INVALID_SRC_ID;
			       for(k = 0 ; k < Nfreq ; k++){
				 	for(m = 0 ; m < Nmic ; m++){
				    		m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[ii+m*Npos_all+k*Nmic*Npos_all].re = 0.0 ;
				    		m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[ii+m*Npos_all+k*Nmic*Npos_all].im = 0.0;
		   	    		}
			  	}
			}
		 }
		 else{
		   // 2010.8.11
		   cout << "Export file : " << cFilename <<" src = " << m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_idPos << endl ;

			MICARY_SMDatabase *pstmpW ;
			// t@CǍ
			MICARY_TFDBFileHeader * psH ;	//f[^x[Xwb_
			psH = (MICARY_TFDBFileHeader *)calloc(1,sizeof(MICARY_TFDBFileHeader)) ;
	
			HA_String file ; // t@C
			file.s = (HA_Char *)calloc(strlen(cFilename)+1,sizeof(char)) ;
			strcpy(file.s,cFilename) ;
			file.slen = strlen(cFilename) ;
	
			if(m_iFormat_ExportW == __FILE_FORMAT_HARK){
				if(!cHark.ReadHeader(file.s,psH)) //t@C̃wb_擾
				{
					cout << "Can not read export file."<< endl ;
					if(psH){
						free(psH) ;
						psH = NULL ;
					}
					if(file.s){
						free(file.s) ;
						file.s = NULL ;
					}
					return false ;
				}
				pstmpW = (MICARY_SMDatabase *)calloc(1,sizeof(MICARY_SMDatabase)) ;	//se|

				SMDatabaseAlloc(pstmpW,0,0,0) ;
				SMDatabaseAlloc(pstmpW,(int)psH->lNFreq,(int)psH->lNSrc,(int)psH->lNMic) ;
			
				// Read Header
				if(!cHark.ReadInfo(file.s,psH ,&pstmpW->micary_locaSource,
						   &pstmpW->micary_locaMic,&pstmpW->micary_smSeparation.micary_iaFreqLinex,false))
				{
					cout << "Can not read export file."<< endl ;
					if(psH){
						free(psH) ;
						psH = NULL ;
					}
					if(file.s){
						free(file.s) ;
						file.s = NULL ;
					}
					SMDatabaseFree(pstmpW) ;
					free(pstmpW) ;
					pstmpW = NULL ;

					return false ;
				}
				Npos = pstmpW->micary_locaSource.ha_numLocation ;
				Nmic = pstmpW->micary_locaMic.ha_numLocation ;
				Nfreq = pstmpW->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;

				if(!cHark.ReadData(file.s,&pstmpW->micary_smSeparation.micary_cmpaSeparation)){
					std ::cout << "Fail to read export file." << std::endl ;
					if(psH){
						free(psH) ;
						psH = NULL ;
					}
					if(file.s){
						free(file.s) ;
						file.s = NULL ;
					}
					SMDatabaseFree(pstmpW) ;
					free(pstmpW) ;
					pstmpW = NULL ;

					return false ;
				}
			
			}
			// ID΁AXV
			bUpdate = false ;

		      idtmp = GetNearestPos(m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii],&pstmpW->micary_locaSource) ;
			if(idtmp != __INVALID_SRC_ID){
				bool btmp = _IsThreshPos(m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii],
								pstmpW->micary_locaSource.micary_Location[idtmp]) ;
				if(btmp){
				  /*
				  cout << "Current Pos = " << m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_idPos << " , " <<  m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_fX << " , " <<  m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_fY << " , " <<  m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_fZ << endl ;
				   cout << "DBfile Pos = " << pstmpW->micary_locaSource.micary_Location[idtmp].ha_idPos << " , " <<  pstmpW->micary_locaSource.micary_Location[idtmp].ha_fX << " , " <<  pstmpW->micary_locaSource.micary_Location[idtmp].ha_fY << " , " <<  pstmpW->micary_locaSource.micary_Location[idtmp].ha_fZ << endl ;
				  */

				//s̈p
			  		for(k = 0 ; k < Nfreq ; k++){
			    			for(m = 0 ; m < Nmic ; m++){
			      			pstmpW->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[idtmp+m*Npos+k*Nmic*Npos].re 
										= m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[ii+m*Npos_all+k*Nmic*Npos_all].re ;
		   	      			pstmpW->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[idtmp+m*Npos+k*Nmic*Npos].im 
										= m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[ii+m*Npos_all+k*Nmic*Npos_all].im ;
		   	    			}
			  		}
			  		//SrcPosXV
			  		pstmpW->micary_locaSource.micary_Location[idtmp].ha_fX = m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_fX ;
			  		pstmpW->micary_locaSource.micary_Location[idtmp].ha_fY = m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_fY ;
			  		pstmpW->micary_locaSource.micary_Location[idtmp].ha_fZ = m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_fZ ;
			  		bUpdate = true ;
				}
			}
		
			if(bUpdate){
				if(m_iFormat_ExportW == __FILE_FORMAT_HARK){
				  psH->lFs = (int)psPrmFrame->ha_fFs ;
				  psH->lNFFT = psPrmFrame->ha_numNFFT ;
				 // 2010.8.11
				  psH->lNSrc = pstmpW->micary_locaSource.ha_numLocation ;
				  psH->lNMic =  pstmpW->micary_locaMic.ha_numLocation ;
				  psH->lNFreq = pstmpW->micary_smSeparation.micary_iaFreqLinex.ha_numValue ;
				  psH->iVersion = 1 ;
				  psH->iFileType = FORMAT_TYPE_W ;
			
				  /*	 
					cout << "File info : Nmic = " << psH->lNMic << " Npos = " << psH->lNSrc << " Nfreq = " << psH->lNFreq << " Version = " << psH->iVersion << " FileType = " << psH->iFileType << endl ;
				  */
				 


					if(!cHark.WriteData(file.s,psH,
							pstmpW->micary_smSeparation.micary_cmpaSeparation,
							pstmpW->micary_smSeparation.micary_iaFreqLinex,
							pstmpW->micary_locaSource,
							pstmpW->micary_locaMic,
							"",
							"",
							"",
							"",
							"",
							    "GHDSS_EXPORT")){
						if(psH){
							free(psH) ;
							psH = NULL ;
						}
						if(file.s){
							free(file.s) ;
							file.s = NULL ;
						}
						SMDatabaseFree(pstmpW) ;
						free(pstmpW) ;
						pstmpW = NULL ;

						return false ;
					}				
				}
			}
			else{	//XV鉹ȂΒǉ
				MICARY_SMDatabase *pstmpW2 ;
				pstmpW2 = (MICARY_SMDatabase *)calloc(1,sizeof(MICARY_SMDatabase)) ;	//se|
				SMDatabaseAlloc(pstmpW2,0,0,0) ;
				SMDatabaseAlloc(pstmpW2,Nfreq,Npos+1,Nmic) ;

				//}CNW
				for(jj = 0 ; jj < Nmic ; jj++){
					pstmpW2->micary_locaMic.ha_idLocation[jj] = 
						pstmpW->micary_locaMic.ha_idLocation[jj] ;
					pstmpW2->micary_locaMic.micary_Location[jj].ha_fX = 
						pstmpW->micary_locaMic.micary_Location[jj].ha_fX ;

					pstmpW2->micary_locaMic.micary_Location[jj].ha_fY = 
						pstmpW->micary_locaMic.micary_Location[jj].ha_fY ;

					pstmpW2->micary_locaMic.micary_Location[jj].ha_fZ = 
						pstmpW->micary_locaMic.micary_Location[jj].ha_fZ ;

					pstmpW2->micary_locaMic.micary_Location[jj].ha_idPos = 
						pstmpW->micary_locaMic.micary_Location[jj].ha_idPos ;

				}

				//gC
				for(jj = 0 ; jj < Nfreq ; jj++){
					pstmpW2->micary_smSeparation.micary_iaFreqLinex.ha_iValue[jj] = 
						pstmpW->micary_smSeparation.micary_iaFreqLinex.ha_iValue[jj] ;
				}

				//W
				for(jj = 0 ; jj < Npos ; jj++){
					pstmpW2->micary_locaSource.ha_idLocation[jj] = 
						pstmpW->micary_locaSource.ha_idLocation[jj] ;
					pstmpW2->micary_locaSource.micary_Location[jj].ha_fX = 
						pstmpW->micary_locaSource.micary_Location[jj].ha_fX ;
					pstmpW2->micary_locaSource.micary_Location[jj].ha_fY = 
						pstmpW->micary_locaSource.micary_Location[jj].ha_fY ;
					pstmpW2->micary_locaSource.micary_Location[jj].ha_fZ = 
						pstmpW->micary_locaSource.micary_Location[jj].ha_fZ ;
					pstmpW2->micary_locaSource.micary_Location[jj].ha_idPos = 
						pstmpW->micary_locaSource.micary_Location[jj].ha_idPos ;

				}
	
				pstmpW2->micary_locaSource.ha_idLocation[Npos] = 
					m_updateW->micary_dbSeparation.micary_locaSource.ha_idLocation[ii] ;
				pstmpW2->micary_locaSource.micary_Location[Npos].ha_fX = 
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_fX ;
				pstmpW2->micary_locaSource.micary_Location[Npos].ha_fY = 
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_fY ;
				pstmpW2->micary_locaSource.micary_Location[Npos].ha_fZ = 
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_fZ ;
				pstmpW2->micary_locaSource.micary_Location[Npos].ha_idPos = 
					m_updateW->micary_dbSeparation.micary_locaSource.micary_Location[ii].ha_idPos ;
	
				/*	
				cout << "Add Pos = " << pstmpW2->micary_locaSource.micary_Location[Npos].ha_idPos << " , " <<  pstmpW2->micary_locaSource.micary_Location[Npos].ha_fX << " , " <<  pstmpW2->micary_locaSource.micary_Location[Npos].ha_fY << " , " <<  pstmpW2->micary_locaSource.micary_Location[Npos].ha_fZ << endl ;
				*/
 
				//t@Cf[^ǂݍ(2009/8/5)
				for(jj = 0 ; jj < Npos +1 ; jj++){								
					for(k = 0 ; k < Nfreq ; k++){
						for(m = 0 ; m < Nmic ; m++){
							if(jj < Npos){
								pstmpW2->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[jj+m*(Npos+1)+k*Nmic*(Npos+1)].re 
									= pstmpW->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[jj+m*Npos+k*Nmic*Npos].re ;
		   						pstmpW2->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[jj+m*(Npos+1)+k*Nmic*(Npos+1)].im 
									= pstmpW->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[jj+m*Npos+k*Nmic*Npos].im ;
							
							}
							else{
								pstmpW2->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[jj+m*(Npos+1)+k*Nmic*(Npos+1)].re 
									= m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[ii+m*Npos_all+k*Nmic*Npos_all].re ;
		   						pstmpW2->micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[jj+m*(Npos+1)+k*Nmic*(Npos+1)].im 
									= m_updateW->micary_dbSeparation.micary_smSeparation.micary_cmpaSeparation.ha_cmpValue[ii+m*Npos_all+k*Nmic*Npos_all].im ;
							}
						}
					}
				}

				// Export File 
				if(m_iFormat_ExportW == __FILE_FORMAT_HARK){
				  psH->lFs = (int)psPrmFrame->ha_fFs ;
				  psH->lNFFT = psPrmFrame->ha_numNFFT ;
				  // 2010.8.11
				  psH->lNSrc = pstmpW2->micary_locaSource.ha_numLocation ;
				  psH->lNMic =  pstmpW2->micary_locaMic.ha_numLocation ;
				  psH->lNFreq = pstmpW2->micary_smSeparation.micary_iaFreqLinex.ha_numValue ; 
				  psH->iVersion = 1 ;
				  psH->iFileType = FORMAT_TYPE_W ;
				  
				  /*
				cout << "File info : Nmic = " << psH->lNMic << " Npos = " << psH->lNSrc << " Nfreq = " << psH->lNFreq << " Version = " << psH->iVersion << " FileType = " << psH->iFileType << endl ;
				  */


				  if(!cHark.WriteData(file.s,psH,
					pstmpW2->micary_smSeparation.micary_cmpaSeparation,
					pstmpW2->micary_smSeparation.micary_iaFreqLinex,
					pstmpW2->micary_locaSource,
					pstmpW2->micary_locaMic,
					"",
					"",
					"",
					"",
					"",
					"GHDSS_EXPORT")){
					if(psH){
					  free(psH) ;
					  psH = NULL ;
					}
					if(file.s){
					  free(file.s) ;
					  file.s = NULL ;
					}
					SMDatabaseFree(pstmpW) ;
					free(pstmpW) ;
					pstmpW = NULL ;
					
				        SMDatabaseFree(pstmpW2) ;
					free(pstmpW2) ;
					pstmpW2 = NULL ;
						cout << "Fail to Export file as HARK2 format." << endl ;
					return false ;
				}
			
			     }


				SMDatabaseFree(pstmpW2) ;
				free(pstmpW2) ;
				pstmpW2 = NULL ;

			}
			//tO̍XV
			if(m_iUpdateMethodW == __UPDATE_METHOD_ID){
				m_updateW->micary_iaUpdate.ha_iValue[ii]=__INVALID_SRC_ID;
			}

			if(psH){
				free(psH) ;
				psH = NULL ;
			}
			if(file.s){
				free(file.s) ;
				file.s = NULL ;
			}
			SMDatabaseFree(pstmpW) ;
			free(pstmpW) ;
			pstmpW = NULL ;
			
		  }
		}		
	}
	
}
//--------------------------------------------------------------------------
bool Cexec::ExportAllUpdateW(MICARY_PrmFrame *psPrmFrame,const char *cFilename)
{
	int Npos_all,ii ;
	bool bRtn ;

	Npos_all = m_updateW->micary_dbSeparation.micary_locaSource.ha_numLocation;
	for(ii = 0 ; ii < Npos_all ;ii++){
		if(m_updateW->micary_iaUpdate.ha_iValue[ii] == __SRCID_FRONT_CREATE	|| 
			m_updateW->micary_iaUpdate.ha_iValue[ii] == __SRCID_FRONT_EXIST)
		{
			m_updateW->micary_iaUpdate.ha_iValue[ii] = __SRCID_FRONT_REMOVE ;
		
		}
	}

	bRtn = AddUpdateW(psPrmFrame,cFilename,true) ;
	return bRtn ;
}
void Cexec::SetTF_CONJ_FileFormat(int iFormat)
{
	m_iFormat_TF_CONJ = iFormat ;
}
void Cexec::SetInitW_FileFormat(int iFormat)
{
	m_iFormat_InitW = iFormat ;
}
void Cexec::SetExportW_FileFormat(int iFormat)
{
	m_iFormat_ExportW = iFormat ;
}
void Cexec::SetMicPos_FileFormat(int iFormat)
{
	m_iFormat_MicPos = iFormat ;
}
void Cexec::SetFixedNoisePos_FileFormat(int iFormat)
{
	m_iFormat_FixedNoise = iFormat ;
}

