// ---------------------------------------------------------------------
/*
 * Copyright 2009 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */
// ---------------------------------------------------------------------
#include <complex>
#include "BufferedNode.h" 
#include "Matrix.h"
#include "Map.h"
#include "Vector.h"
// ---------------------------------------------------------------------

// namespacegp錾邱
using namespace FD;
using namespace std;

//forward declaration of class VAdd for use with the DECLARE_NODE macro
class PowerCalcForMatrix; // m[h

//Declaration of the node. This definition is transformed into XML data for the GUI, as well as documentation for the node
// m[h̓o͒[q쐬
DECLARE_NODE(PowerCalcForMatrix) // m[h

/*Node
 *  
 * @name PowerCalcForMatrix
 * @category HARK:MISC
 * @description Calculate power spectrum from complex spectrum for Matrix data.
 *
 * @input_name INPUT
 * @input_type Matrix<complex<float> >
 * @input_description Complex spectrum. Row: source ID, Col: Data for each source.
 *
 * @output_name OUTPUT
 * @output_type Matrix<float>
 * @output_description Power spectrum. Row: source ID, Col: Data for each source.
 * 
 * @parameter_name POWER_TYPE
 * @parameter_type string
 * @parameter_list POW:MAG
 * @parameter_value POW
 * @parameter_description Measure for computing the POW or MAG (i.e. power or magnitude) of the complex spectrum [default: POW]
 * 
END*/

// ---------------------------------------------------------------------

//Class definition/implementation. Note that because we won't need to derive from this class, we don't need a header file (.h) and we can put everything in the .cc. Our node, like most other nodes, derives from BufferedNode.

class PowerCalcForMatrix : public BufferedNode {

  int m_iInputID;
  int m_iOutputID;
  string powerType;

public:
  PowerCalcForMatrix(string nodeName, ParameterSet params)
    : BufferedNode(nodeName, params)
  { 

    //In the constructor, we create both the inputs and outputs.
		// 2008/08/19 changed by N.Tanaka
		/*
    try {
      m_iInputID = addInput("INPUT");
      m_iOutputID = addOutput("OUTPUT");
    }
    catch(BaseException *e){
      e->add(new NodeException(NULL, "Exception caught in PowerCalcForMatrix constructor", __FILE__, __LINE__));
    }
		*/
    m_iInputID = addInput("INPUT");
    m_iOutputID = addOutput("OUTPUT");

    powerType = object_cast<String>(parameters.get("POWER_TYPE"));        
  } 


	//This is the main method for the node, it is called from the BufferedNode class each time a result needs to be calculated.
  void calculate(int output_id, int count, Buffer &out)
  {
    
    int iNRow, iNCol;
    int i, k;
    float fTemp;
    complex<float> cTemp;
    ObjectRef orInputTemp, orOutputTemp, orOutputTemp2;
    Map<int, ObjectRef>::iterator pmTemp;
    
	  
		// 2008/08/19 deleted by N.Tanaka
		// BaseExceptioncatchCflowdesignerɂ܂
//    try {
      // Input
			// 2008/08/19 changed by N.Tanaka
      //ObjectRef orInputTemp = getInput(m_iInputID, count);
      orInputTemp = getInput(m_iInputID, count);

      if (typeid(*orInputTemp)==typeid(Matrix<complex<float> >)){// Matrix
	Matrix<complex<float> > &mInputValue = object_cast<Matrix<complex<float> > > (orInputTemp);

	iNRow = mInputValue.nrows();
	iNCol = mInputValue.ncols();
		


	// Output
	orOutputTemp = ObjectRef(new Matrix<float>(iNRow, iNCol));
	Matrix<float> &mOutputValue = object_cast<Matrix<float> >(orOutputTemp);
	out[count] = orOutputTemp; 
      
	// norm
	for (i = 0 ; i < iNRow ; i++){
	  for (k = 0 ; k < iNCol ; k++){
	  
	    cTemp = mInputValue(i, k);

	    if(powerType == "POW")
	    	fTemp = norm(cTemp);
	    else if(powerType == "MAG")
		fTemp = abs(cTemp);

	    mOutputValue(i, k) = fTemp;			
	  }
	}
      }
      else if(typeid(*orInputTemp)==typeid(Map<int, ObjectRef>)) {// Map<int, ObjectRef>
	/*
	// Input
	Map<int, ObjectRef> &mpInputValue = object_cast<Map<int, ObjectRef> >(orInputTemp);

	// Output
	orOutputTemp = ObjectRef(new Map<int, ObjectRef>);
	Map<int, ObjectRef> &mpOutputValue = object_cast<Map<int, ObjectRef> >(orOutputTemp);
	out[count] = orOutputTemp;

	mpOutputValue.clear();

	// norm
	for (pmTemp = mpInputValue.begin() ; pmTemp != mpInputValue.end() ; pmTemp++){
	  orInputTemp = pmTemp->second;
	  Vector<complex<float> > &vcfInputTemp = object_cast<Vector<complex<float> > >(orInputTemp);

	  orOutputTemp2 = ObjectRef(Vector<float>::alloc(vcfInputTemp.size()));
	  Vector<float> &vfOutputTemp = object_cast<Vector<float> >(orOutputTemp2);

	  for (i = 0 ; i < vcfInputTemp.size() ; i++){
	    vfOutputTemp[i] = norm(vcfInputTemp[i]);
	  }

	  mpOutputValue.insert(make_pair(pmTemp->first, orOutputTemp2));
	  
	}
	*/
      }
//    }
//    catch (BaseException *e){
//      e->add(new NodeException(NULL, "Exception caught in PowerCalcForMatrix calculate", __FILE__, __LINE__));
//    }


  }

};
